import type { Constructor, Nullable, Predicate } from '../types.js';
import type { SortFunction, SortOrder, TransformTo1DArray } from './index.js';
import PotentIterator from './PotentIterator.js';
import type ReadonlyList from './ReadonlyList.js';
/**
 * Extends the built-in Array object with additional convenience methods.
 *
 * Note that these methods can be applied to standard arrays by using their
 * free function variants in the `Arrays` module.
 */
export default class List<T> extends Array<T> implements ReadonlyList<T> {
    get(index: number): T;
    clone(): List<T>;
    first(): T | undefined;
    last(): T | undefined;
    lastIndex(): number;
    isInBounds(index: number): boolean;
    /**
     * @example
     * const list = new List(5);  // contains 5 empty "slots"
     * list.isInBounds(2);        // -> true
     * list.hasElementAt(2);      // -> false
     */
    hasElementAt(index: number): boolean;
    nextIndex(fromIndex: number): number;
    next(fromIndex: number): T | undefined;
    previousIndex(fromIndex: number): number;
    previous(fromIndex: number): T | undefined;
    isEmpty(): boolean;
    isNotEmpty(): boolean;
    equals(array2: readonly unknown[]): boolean;
    /**
     * Removes the specified element(s) from the list and returns the list itself.
     *
     * If the list contains multiples of the specified element(s), all of them
     * are removed.
     *
     * If the list does not contain one of the elements, throws an error.
     */
    remove(...elements: T[]): this;
    /**
     * Removes the element at the specified index from the list and returns the
     * list itself.
     *
     * @param index The zero-based index of the element to be removed.
     *              If negative, iterates backwards from the end of the list.
     */
    removeAt(index: number): this;
    /**
     * Replaces the specified element with another and returns this list.
     *
     * If the list contains multiples of the target element,
     * it will only replace the first occurrence.
     */
    replace(element: T, replacement: T): this;
    /**
     * Replaces all elements in the list with those in the iterable.
     */
    replaceAll(replacement: Iterable<T>): this;
    /**
     * Inserts an element at the given index.
     */
    insert(index: number, ...elements: T[]): this;
    type<T>(type: Constructor<T>): this is List<T>;
    type<T>(type: Constructor<T>): this is ReadonlyList<T>;
    type(type: 'bigint'): this is List<T>;
    type(type: 'bigint'): this is ReadonlyList<bigint>;
    type(type: 'function'): this is List<Function>;
    type(type: 'function'): this is ReadonlyList<Function>;
    type(type: 'object'): this is Array<Record<string, unknown> | null>;
    type(type: 'object'): this is ReadonlyList<Record<string, unknown> | null>;
    type(type: 'symbol'): this is List<symbol>;
    type(type: 'symbol'): this is ReadonlyList<symbol>;
    type(type: 'undefined'): this is List<undefined>;
    type(type: 'undefined'): this is ReadonlyList<undefined>;
    type(type: 'boolean'): this is List<boolean>;
    type(type: 'boolean'): this is ReadonlyList<boolean>;
    type(type: 'number'): this is List<number>;
    type(type: 'number'): this is ReadonlyList<number>;
    type(type: 'string'): this is List<string>;
    type(type: 'string'): this is ReadonlyList<string>;
    /**
     * Sorts the list in the standard way according to the data type contained within.
     *
     * This function is meant to be a shorthand to avoid having to type out a sorting
     * function every time and remember which of the two arguments subtracts from which.
     *
     * | Data type    | Sorting algorithm |
     * | ------------ | ----------------- |
     * | `string`     | Alphabetical      |
     * | `number`     | Hexadecimal       |
     * | `Date`       | Chronological     |
     * | any other    | according to stringified code point value |
     *
     * @param order The order to sort in. Possible values are 'descending' (largest first)
     * and 'ascending' (smallest first). Default is 'ascending'.
     */
    sort(order: SortOrder): this;
    sort(): this;
    /**
     * Sorts the list by one or more sort functions. Later sort functions will only be used
     * if their predecessor does not return a conclusive result (i.e. returns zero).
     *
     * Sort functions follow the `Array.prototype.sort()` schema:
     *
     * * If it returns a number < 0, `a` comes before `b`.
     * * If it returns a number > 0, `b` comes before `a`.
     * * If it returns 0, the next sort function is used instead. If no sort functions remain,
     * the order is unchanged.
     *
     * In practice, this usually means that your sort function should be `(a, b) => a - b`
     * if you want an ascending sort (i.e. smallest element first) and `(a, b) => b - a`
     * if you want a descending sort (i.e. largest element first).
     */
    sort(...sortFns: SortFunction<T>[]): this;
    /** Clears all elements from the list and returns it. */
    clear(): this;
    /** Removes all null or undefined elements from the list and returns it. */
    clearNull(): this;
    /** Swaps the elements at the specified indices. */
    swap(index1: number, index2: number): this;
    closest(callback: (item: T) => number, target: number): T;
    /**
     * Moves each array item by `by`. This method will wrap the array.
     * As a result, there will never be a negative or unfilled index.
     */
    moveAll(by: number): this;
    /**
     * @example new List(0, 1, 2).zip([4, 5, 6]) => [[0, 4], [1, 5], [2, 6]]
     */
    zip<Args extends Array<ReadonlyArray<unknown>>>(...arrays: Args): List<[T, ...TransformTo1DArray<Args>]>;
    filterMap<U>(mapFn: (object: T) => Nullable<U>): List<U>;
    correlate<B, C, D, E, F, G, H, I, J>(source2: readonly B[], source3: readonly C[], source4: readonly D[], source5: readonly E[], source6: readonly F[], source7: readonly G[], source8: readonly H[], source9: readonly I[], source10: readonly J[], callback: (a: T, b: B, c: C, d: D, e: E, f: F, g: G, h: H, i: I, j: J) => void): void;
    correlate<B, C, D, E, F, G, H, I>(source2: readonly B[], source3: readonly C[], source4: readonly D[], source5: readonly E[], source6: readonly F[], source7: readonly G[], source8: readonly H[], source9: readonly I[], callback: (a: T, b: B, c: C, d: D, e: E, f: F, g: G, h: H, i: I) => void): void;
    correlate<B, C, D, E, F, G, H>(source2: readonly B[], source3: readonly C[], source4: readonly D[], source5: readonly E[], source6: readonly F[], source7: readonly G[], source8: readonly H[], callback: (a: T, b: B, c: C, d: D, e: E, f: F, g: G, h: H) => void): void;
    correlate<B, C, D, E, F, G>(source2: readonly B[], source3: readonly C[], source4: readonly D[], source5: readonly E[], source6: readonly F[], source7: readonly G[], callback: (a: T, b: B, c: C, d: D, e: E, f: F, g: G) => void): void;
    correlate<B, C, D, E, F>(source2: readonly B[], source3: readonly C[], source4: readonly D[], source5: readonly E[], source6: readonly F[], callback: (a: T, b: B, c: C, d: D, e: E, f: F) => void): void;
    correlate<B, C, D, E>(source2: readonly B[], source3: readonly C[], source4: readonly D[], source5: readonly E[], callback: (a: T, b: B, c: C, d: D, e: E) => void): void;
    correlate<B, C, D>(source2: readonly B[], source3: readonly C[], source4: readonly D[], callback: (a: T, b: B, c: C, d: D) => void): void;
    correlate<B, C>(source2: readonly B[], source3: readonly C[], callback: (a: T, b: B, c: C) => void): void;
    correlate<B>(source2: readonly B[], callback: (a: T, b: B) => void): void;
    groupBy(property: (item: T) => unknown): T[][];
    groupBy<U, Result>(property: (item: T) => U, mapGroup: (property: U, items: readonly T[]) => Result): Result[];
    distinct(): List<T>;
    hasDuplicates(): boolean;
    hasMultiple(value: T): boolean;
    findIndices(predicate: Predicate<T>): List<number>;
    findIndices(object: T): List<number>;
    union(...arrays: readonly T[][]): List<T>;
    intersection(...arrays: readonly T[][]): List<T>;
    difference(...arrays: readonly T[][]): List<T>;
    iterator(): PotentIterator<T>;
    toJSON(): T[];
    /**
     * Creates an array from an iterable object.
     * @param iterable An iterable object to convert to an array.
     * @param mapfn A mapping function to call on every element of the array.
     * @param thisArg Value of 'this' used to invoke the mapfn.
     */
    static from<T, U = T>(iterable: Iterable<T> | ArrayLike<T>, mapfn?: (v: T, k: number) => U, thisArg?: unknown): List<U>;
    /**
     * Returns a new array from a set of elements.
     * @param items A set of elements to include in the new array object.
     */
    static of<T>(...items: T[]): List<T>;
}
