import * as Arrays from '../arrays/index.js';
import { swap } from '../objects/index.js';
import PotentIterator from './PotentIterator.js';
/**
 * Extends the built-in Array object with additional convenience methods.
 *
 * Note that these methods can be applied to standard arrays by using their
 * free function variants in the `Arrays` module.
 */
export default class List extends Array {
    get(index) {
        if (this.hasElementAt(index)) {
            return this[index];
        }
        throw new Error(`Index was out of bounds at ${index}.`);
    }
    clone() {
        return new List(...this);
    }
    first() {
        return Arrays.first(this);
    }
    last() {
        return Arrays.last(this);
    }
    lastIndex() {
        return Arrays.lastIndex(this);
    }
    isInBounds(index) {
        return Arrays.isInBounds(this, index);
    }
    /**
     * @example
     * const list = new List(5);  // contains 5 empty "slots"
     * list.isInBounds(2);        // -> true
     * list.hasElementAt(2);      // -> false
     */
    hasElementAt(index) {
        return Arrays.hasElementAt(this, index);
    }
    nextIndex(fromIndex) {
        return Arrays.nextIndex(this, fromIndex);
    }
    next(fromIndex) {
        return Arrays.next(this, fromIndex);
    }
    previousIndex(fromIndex) {
        return Arrays.previousIndex(this, fromIndex);
    }
    previous(fromIndex) {
        return Arrays.previous(this, fromIndex);
    }
    isEmpty() {
        return Arrays.isEmpty(this);
    }
    isNotEmpty() {
        return Arrays.isNotEmpty(this);
    }
    equals(array2) {
        return Arrays.equal(this, array2);
    }
    /**
     * Removes the specified element(s) from the list and returns the list itself.
     *
     * If the list contains multiples of the specified element(s), all of them
     * are removed.
     *
     * If the list does not contain one of the elements, throws an error.
     */
    remove(...elements) {
        Arrays.remove(this, ...elements);
        return this;
    }
    /**
     * Removes the element at the specified index from the list and returns the
     * list itself.
     *
     * @param index The zero-based index of the element to be removed.
     *              If negative, iterates backwards from the end of the list.
     */
    removeAt(index) {
        Arrays.removeAt(this, index);
        return this;
    }
    /**
     * Replaces the specified element with another and returns this list.
     *
     * If the list contains multiples of the target element,
     * it will only replace the first occurrence.
     */
    replace(element, replacement) {
        Arrays.replace(this, element, replacement);
        return this;
    }
    /**
     * Replaces all elements in the list with those in the iterable.
     */
    replaceAll(replacement) {
        Arrays.replaceAll(this, replacement);
        return this;
    }
    /**
     * Inserts an element at the given index.
     */
    insert(index, ...elements) {
        Arrays.insert(this, index, ...elements);
        return this;
    }
    type(type) {
        // Overloaded forwarding still hasn't been implemented
        // in TypeScript, so we have to make do.
        return Arrays.type(this, type);
    }
    sort(orderOrSortFn = 'ascending', ...sortFns) {
        if (typeof orderOrSortFn === 'function' && sortFns.length === 0) {
            // Without this branch we'll have infinite recursion on our hands
            // as Arrays.sort() calls Array.prototype.sort(), which will call
            // List.sort() on Lists, which would call Arrays.sort() again.
            return super.sort(orderOrSortFn);
        }
        try {
            Arrays.sort(this, orderOrSortFn, ...sortFns);
        }
        catch (_a) {
            // Original function fails if type isn't string, Date, or number.
            super.sort();
        }
        return this;
    }
    /** Clears all elements from the list and returns it. */
    clear() {
        Arrays.clear(this);
        return this;
    }
    /** Removes all null or undefined elements from the list and returns it. */
    clearNull() {
        Arrays.clearNull(this);
        return this;
    }
    /** Swaps the elements at the specified indices. */
    swap(index1, index2) {
        swap(this, index1, index2);
        return this;
    }
    closest(callback, target) {
        return Arrays.closest(this, callback, target);
    }
    /**
     * Moves each array item by `by`. This method will wrap the array.
     * As a result, there will never be a negative or unfilled index.
     */
    moveAll(by) {
        Arrays.moveAll(this, by);
        return this;
    }
    /**
     * @example new List(0, 1, 2).zip([4, 5, 6]) => [[0, 4], [1, 5], [2, 6]]
     */
    zip(...arrays) {
        return List.from(Arrays.zip(this, ...arrays));
    }
    filterMap(mapFn) {
        return Arrays.filterMap(this, mapFn);
    }
    correlate(...args) {
        // @ts-expect-error  TypeScript cannot "see" the base function, so it will always throw an error.
        return Arrays.correlate(this, ...args);
    }
    groupBy(property, mapGroup) {
        return Arrays.groupBy(this, property, mapGroup);
    }
    distinct() {
        return List.from(new Set(this));
    }
    hasDuplicates() {
        return Arrays.hasDuplicates(this);
    }
    hasMultiple(value) {
        return Arrays.hasMultiple(this, value);
    }
    findIndices(objectOrPredicate) {
        return List.from(Arrays.findIndices(this, objectOrPredicate));
    }
    union(...arrays) {
        return List.from(Arrays.union(this, ...arrays));
    }
    intersection(...arrays) {
        return List.from(Arrays.intersection(this, ...arrays));
    }
    difference(...arrays) {
        return List.from(Arrays.difference(this, ...arrays));
    }
    iterator() {
        return new PotentIterator(this);
    }
    toJSON() {
        return [...this];
    }
    /**
     * Creates an array from an iterable object.
     * @param iterable An iterable object to convert to an array.
     * @param mapfn A mapping function to call on every element of the array.
     * @param thisArg Value of 'this' used to invoke the mapfn.
     */
    static from(iterable, mapfn, thisArg) {
        return super.from(iterable, mapfn, thisArg);
    }
    /**
     * Returns a new array from a set of elements.
     * @param items A set of elements to include in the new array object.
     */
    static of(...items) {
        return super.of(...items);
    }
}
