import type { ArrayType, TupleType } from '../types.js';
declare const addKeys: readonly ["push", "add", "enqueue", "attach"];
declare type Collection<T> = _Collection<T, TupleType<typeof addKeys>>;
declare type _Collection<T, U> = U extends string ? {
    [K in U]: (element: T) => unknown;
} : never;
/**
 * Iterator that expands the built-in {@link Iterator} with powerful utility
 * iteration methods that consume or advance the iterator and promotes
 * iterator-centric (as opposed to collection-centric) design.
 *
 * This is a *wrapper* for existing iterators. Existing iterators are not
 * mutated, but they _will_ be consumed by the methods in this class.
 * Any built-in iterator can be wrapped in a PotentIterator.
 *
 * This class distinguishes between *consumer* and *producer* methods.
 * A *consumer* method immediately consumes the underlying iterator and returns
 * an aggregated result.
 * A *producer* method does not consume the underlying
 * iterator immediately; instead it creates a new iterator that operates on
 * and consumes the underlying iterator when it is itself consumed.
 *
 * All methods that modify the iterated sequence are producer methods, and
 * thus lazily executed. This allows you to chain function calls without
 * worrying about redundant iterations.
 * For instance, in a standard JavaScript array,
 * `strings.filter(x => x.startsWith('/')).map(x => x.slice(1))`
 * would iterate over the entire array twice. Using PotentIterator, the
 * same function calls will only iterate over the array once.
 *
 * Note that similar methods as encapsulated by this class are currently
 * a [TC39 candidate](https://github.com/tc39/proposal-iterator-helpers).
 * If the native built-in iterator object is extended by these functions,
 * this class will be marked as deprecated and removed or simplified
 * in a future release of *potence*.
 */
export default class PotentIterator<T> implements IterableIterator<T> {
    next: (...args: [] | [undefined]) => IteratorResult<T, undefined>;
    return?: (value?: unknown) => IteratorResult<T, undefined>;
    throw?: (e?: unknown) => IteratorResult<T, undefined>;
    /**
     * Creates a new iterator based on an existing iterator or iterable.
     * Note that _no copy_ will be created of the underlying iterator; this
     * iterator operates directly on the underlying iterator, consuming it in
     * the process.
     */
    constructor(iterator: Iterator<T> | Iterable<T>);
    [Symbol.iterator](): PotentIterator<T>;
    /**
     * Consumes the first element of the iterator and returns it,
     * or `undefined` if the sequence was empty.
     */
    first(): T | undefined;
    /**
     * Consumes the iterator and returns the last element,
     * or `undefined` if the sequence was empty.
     */
    last(): T | undefined;
    /**
     * Returns the `n`th element in the iterator,
     * or `undefined` if there is no element at that position.
     * This consumes the returned element as well as all
     * preceding elements in the iterator.
     *
     * This function is zero-indexed.
     */
    nth(n: number): T | undefined;
    /**
     * Consumes the iterator until an element is found that matches the
     * predicate, then returns it.
     *
     * Note that this method will consume the entire iterator only if no
     * matching element is found.
     */
    find(predicate: (value: T, index: number) => boolean): T | undefined;
    /**
     * Consumes the iterator and returns the last element that matches
     * the predicate.
     *
     * This method will always consume the entire iterator.
     */
    findLast(predicate: (value: T, index: number) => boolean): T | undefined;
    /**
     * Consumes the iterator until an element is found that matches the
     * predicate, then returns its index.
     *
     * Note that this method will consume the entire iterator only if no
     * matching element is found.
     *
     * If no element is found that matches the predicate, returns `-1`.
     */
    findIndex(predicate: (value: T, index: number) => boolean): number;
    /**
     * Consumes the iterator and returns the index of the last element that
     * matches the predicate.
     *
     * This method will always consume the entire iterator.
     *
     * If no element is found that matches the predicate, returns `-1`.
     */
    findLastIndex(predicate: (value: T, index: number) => boolean): number;
    /**
     * Consumes the iterator and executes the provided callback function
     * for every element in the sequence.
     *
     * Unless you already have a predefined callback function (like
     * `iterator.forEach(myFunc)`) or your callback function is a one-liner
     * it is generally recommended to use a standard `for` loop instead.
     */
    forEach(callback: (value: T, i: number) => void): void;
    /**
     * Consumes the iterator and returns `true` if every element matched
     * the predicate, otherwise `false`.
     *
     * If the iterator contains no elements, returns `true`.
     *
     * Note that the iterator is only consumed until an element is encountered
     * which does not match the predicate. If every element matches the
     * predicate, the iterator is fully consumed.
     */
    every(predicate: (value: T) => boolean): boolean;
    /**
     * Consumes the iterator and returns `true` if at least one element matched
     * the predicate, otherwise `false`.
     *
     * If the iterator contains no elements, returns `false`.
     *
     * Note that the iterator is only consumed until an element is encountered
     * which matches the predicate. If no element matches the predicate, the
     * iterator is fully consumed.
     */
    some(predicate: (value: T) => boolean): boolean;
    /**
     * Consumes the iterator and returns an aggregated result by applying
     * a reducer function to each element.
     *
     * @example
     * new PotentIterator([3, 5, 9, 1]).reduce((acc, element) => acc > element ? acc : element, -Infinity);
     * // Output: 9
     */
    reduce<U>(callback: (accumulator: U, element: T, i: number) => U, initialValue: U): U;
    /** Consumes the iterator and returns the remaining number of iterations. */
    count(): number;
    /**
     * Consumes the iterator and returns the sum of all elements.
     *
     * If any of the elements are not coercible to a number, returns `NaN`.
     */
    sum(): number;
    /**
     * Consumes the iterator and returns the largest element.
     *
     * If any of the elements are not coercible to a number or
     * the iterator contains no elements, returns `NaN`.
     */
    max(): number;
    /**
     * Consumes the iterator and returns the smallest element.
     *
     * If any of the elements are not coercible to a number or
     * the iterator contains no elements, returns `NaN`.
     */
    min(): number;
    /** Consumes the iterator and returns all remaining elements as an array. */
    collect(): T[];
    /** Consumes the iterator and adds all remaining elements to the collection. */
    collect<U extends Collection<T>>(into: U): U;
    /**
     * Consumes the iterator and calls `console.log()` on each element.
     * Useful for debug output.
     */
    log(): void;
    /**
     * Creates a new iterator that skips (consuming) the first `n` elements
     * and yields only the remaining elements.
     *
     * If there are less than `n` elements in the iterator, no more elements
     * will be yielded.
     *
     * @example
     * new PotentIterator([0, 1, 2, 3]).skip(2).log();
     * // Output:
     * // 2
     * // 3
     */
    skip(n: number): PotentIterator<T>;
    /**
     * Creates a new iterator that yields at most the first `n` elements of
     * this iterator.
     *
     * If `n` is less than the number of elements in the iterator, the new
     * iterator will instead stop after the last element has been consumed.
     *
     * @example
     * new PotentIterator([0, 1, 2, 3]).take(2).log();
     * // Output:
     * // 0
     * // 1
     */
    take(n: number): PotentIterator<T>;
    /**
     * Creates a new iterator that iterates through this iterator and then
     * the passed iterator consecutively, thus chaining them and effectively
     * extending the number of elements in the iterator by the number of
     * elements in `iterator`.
     *
     * @example
     * new PotentIterator([0, 1, 2]).chain([3, 4, 5]).map(x => -x).log();
     * // Output:
     * // 0
     * // -1
     * // -2
     * // -3
     * // -4
     * // -5
     */
    chain(iterator: Iterator<T> | Iterable<T>): PotentIterator<T>;
    /**
     * Creates a new iterator that combines the two iterators by executing and
     * consuming them side-by-side, returning each element as part of a tuple
     * every iteration.
     * The length of the new iterator will be the length of the smallest of the
     * two iterators. For example, if the first iterator has 5 elements and the
     * second iterator has 7, the new iterator will at most iterate 5 times.
     *
     * @example
     * new PotentIterator([0, 1, 2]).zip(['one', 'two']).log();
     * // Output:
     * // [0, 'one']
     * // [1, 'two']
     */
    zip<U>(iterator: Iterator<U> | Iterable<U>): PotentIterator<[T, U]>;
    /**
     * Creates a new iterator with each element of type `T` mapped to a `U`.
     *
     * @example
     * new PotentIterator([0, 1, 2]).map(x => x * 2).log();
     * // Output:
     * // 0
     * // 2
     * // 4
     */
    map<U>(callback: (value: T, index: number) => U): PotentIterator<U>;
    /**
     * Creates a new flattened iterator, that is: any iterable child elements
     * will be extracted and returned from the new iterator as if they were
     * ordinary iterator elements, and not wrapped in an array. This naturally
     * expands the number of elements this iterator contains.
     *
     * This method only flattens one level of iterables. If you have multiple
     * levels to flatten, you can chain function calls of this method.
     *
     * @example
     * new PotentIterator(['1.23.45']).map(x => x.split('.')).flatten().log();
     * // Output:
     * // 1
     * // 23
     * // 45
     */
    flatten(): PotentIterator<ArrayType<T>>;
    /**
     * Creates a new iterator with elements filtered that do not match the
     * predicate.
     *
     * @example
     * new PotentIterator([-4, 2, -9, 1]).filter(x => x >= 0).log();
     * // Output:
     * // 2
     * // 1
     */
    filter(predicate: (value: T, index: number) => boolean): PotentIterator<T>;
    /**
     * Creates a new iterator where each element is a tuple composed of the
     * original element and its original index. Useful to keep the information
     * about the original index if you intend to constrain the sequence.
     *
     * @example
     * new PotentIterator([9, 8, 7, 6]).indexed().skip(2).log();
     * // Output:
     * // [7, 2]
     * // [6, 3]
     */
    indexed(): PotentIterator<[T, number]>;
}
export {};
