/**
 * @file Offers convenience methods on arrays.
 *
 * Note that unknown iterations use `Array.prototype.forEach()` and Co.
 * In older browsers, a simple `for` loop was significantly faster.
 * In modern browsers, that is still the case, but the difference
 * is now negligible. Even on arrays with a length of over 10,000,000,
 * the difference in performance makes up barely 30 ms.
 */
import type { ArrayType, Constructor, Nullable } from '../types.js';
/**
 * Returns a copy of the specified array.
 * Note that only the array is copied, not the elements within.
 */
export declare function clone<T>(array: readonly T[]): T[];
interface HasLength {
    length: number;
}
interface HasSize {
    size: number;
}
/**
 * Returns the number of elements in the iterable or array-like.
 *
 * If the iterable contains a `length` property (such as in {@link Array})
 * or a `size` property (such as in {@link Set}), this function will use
 * them to obtain the number of elements in the iterable. Otherwise it
 * will count the number of elements by iterating over the iterable manually.
 */
export declare function count(iterable: Iterable<unknown> | HasLength | HasSize): number;
/**
 * Gets the first element in the iterable,
 * or undefined if the iterable contains no elements.
 */
export declare function first<T>(iterable: Iterable<T>): T | undefined;
/**
 * Gets the index of the last element in the iterable,
 * or `-1` if the iterable contains no elements.
 */
export declare function lastIndex<T>(iterable: Iterable<T>): number;
/**
 * Gets the last element in the iterable,
 * or undefined if the iterable contains no elements.
 */
export declare function last<T>(iterable: Iterable<T>): T | undefined;
/**
 * Returns true if the given index is in-bounds of the array, i.e. greater
 * than 0 and smaller than the array's length.
 *
 * Note that this function does **not** check whether there is actually an
 * array element at the specified index, as in when an array is initialized
 * with empty slots (e.g. using `Array`'s length constructor).
 * Unless you have a good reason to use `isInBounds()`, you probably want to use
 * `Arrays.hasElementAt()` instead.
 */
export declare function isInBounds(array: readonly unknown[], index: number): boolean;
/**
 * Returns true if the given index refers to an actual element on the specified
 * array.
 *
 * @example
 * const array = new Array(5);     // contains 5 empty "slots"
 * Arrays.isInBounds(array, 2);    // -> true
 * Arrays.hasElementAt(array, 2);  // -> false
 */
export declare function hasElementAt(array: readonly unknown[], index: number): boolean;
/**
 * Gets the index of the next element in the array, starting at the given index.
 * If the index belongs to the last element in the array, returns the index of
 * the first element. Returns `-1` if the given index is out of bounds.
 */
export declare function nextIndex(array: readonly unknown[], fromIndex: number): number;
/**
 * Gets the next element in the array, starting at the given index. If the index belongs to the
 * last element in the array, returns the first element. Returns undefined if the given index
 * is out of bounds.
 */
export declare function next<T>(array: readonly T[], fromIndex: number): T | undefined;
/**
 * Gets the index of the previous element in the array, starting at the given
 * index. If the index belongs to the first element in the array, returns the
 * index of the last element. Returns `-1` if the given index is out of bounds.
 */
export declare function previousIndex(array: readonly unknown[], fromIndex: number): number;
/**
 * Gets the previous element in the array, starting at the given index. If the index belongs to the
 * first element in the array, returns the last element. Returns undefined if the given index
 * is out of bounds.
 */
export declare function previous<T>(array: readonly T[], fromIndex: number): T | undefined;
/** Returns true if the given array is empty, otherwise false. */
export declare function isEmpty(iterable: Iterable<unknown> | HasLength | HasSize): boolean;
/** Returns true if the given array is not empty, otherwise false. */
export declare function isNotEmpty(iterable: Iterable<unknown> | HasLength | HasSize): boolean;
/**
 * Compares the contents of two arrays for referential equality and returns true
 * if both arrays have the same elements in the same positions.
 */
export declare function equal(array1: readonly unknown[], array2: readonly unknown[]): boolean;
/**
 * Removes the specified element(s) from the array and returns the array.
 *
 * If the array contains multiples of the specified element(s), all of them
 * are removed.
 *
 * If the array does not contain one of the elements, throws an error.
 */
export declare function remove<T extends unknown[]>(array: T, ...elements: T): T;
/**
 * Removes the element at the specified index from the array
 * and returns the array.
 *
 * @param index The zero-based index of the element to be removed.
 *              If negative, iterates backwards from the end of the array.
 */
export declare function removeAt<T extends unknown[]>(array: T, index: number): T;
/**
 * Replaces the specified element with another and returns the array.
 *
 * If the array contains multiples of the target element, it will only replace the first occurrence.
 */
export declare function replace<TArray extends T[], T>(array: TArray, element: T, replacement: T): TArray;
/**
 * Replaces all elements in the array with all elements from another array.
 */
export declare function replaceAll<TArray extends T[], T>(array: TArray, replacement: Iterable<T>): TArray;
/**
 * Inserts one or multiple elements at the given index.
 *
 * This function throws an error if the index is not in-bounds.
 */
export declare function insert<T extends unknown[]>(array: T, index: number, ...elements: T): T;
/**
 * Checks if the array is composed of the given type and only of the given type.
 *
 * Since this is a type guard, TypeScript users can afterwards use the array
 * as if it were of the specified type without the necessity of an additional cast.
 */
export declare function type<T>(array: unknown[], type: Constructor<T>): array is T[];
export declare function type<T>(array: readonly unknown[], type: Constructor<T>): array is readonly T[];
export declare function type(array: unknown[], type: 'bigint'): array is bigint[];
export declare function type(array: readonly unknown[], type: 'bigint'): array is readonly bigint[];
export declare function type(array: unknown[], type: 'function'): array is Function[];
export declare function type(array: readonly unknown[], type: 'function'): array is readonly Function[];
export declare function type(array: unknown[], type: 'object'): array is Array<Record<string, unknown> | null>;
export declare function type(array: readonly unknown[], type: 'object'): array is ReadonlyArray<Record<string, unknown> | null>;
export declare function type(array: unknown[], type: 'symbol'): array is symbol[];
export declare function type(array: readonly unknown[], type: 'symbol'): array is readonly symbol[];
export declare function type(array: unknown[], type: 'undefined'): array is undefined[];
export declare function type(array: readonly unknown[], type: 'undefined'): array is readonly undefined[];
export declare function type(array: unknown[], type: 'boolean'): array is boolean[];
export declare function type(array: readonly unknown[], type: 'boolean'): array is readonly boolean[];
export declare function type(array: unknown[], type: 'number'): array is number[];
export declare function type(array: readonly unknown[], type: 'number'): array is readonly number[];
export declare function type(array: unknown[], type: 'string'): array is string[];
export declare function type(array: readonly unknown[], type: 'string'): array is readonly string[];
export declare type SortOrder = 'ascending' | 'descending';
export declare type SortFunction<T = unknown> = (a: T, b: T) => number;
/**
 * Sort an array by one or more sort functions. Later sort functions will only be used
 * if their predecessor does not return a conclusive result (i.e. returns zero).
 *
 * Sort functions follow the `Array.prototype.sort()` schema:
 *
 * * If it returns a number < 0, `a` comes before `b`.
 * * If it returns a number > 0, `b` comes before `a`.
 * * If it returns 0, the next sort function is used instead. If no sort functions remain,
 * the order is unchanged.
 *
 * In practice, this usually means that your sort function should be `(a, b) => a - b`
 * if you want an ascending sort (i.e. smallest element first) and `(a, b) => b - a`
 * if you want a descending sort (i.e. largest element first).
 */
export declare function sort<T>(array: T[], ...sortFns: SortFunction<T>[]): T;
/**
 * Sorts the array in the standard way according to the data type contained within.
 * Unsupported data types (like object literals or arrays) will throw an error.
 *
 * This function is meant to be a shorthand to avoid having to type out a sorting
 * function every time and remember which of the two arguments subtracts from which.
 *
 * | Data type    | Sorting algorithm |
 * | ------------ | ----------------- |
 * | `string`     | Alphabetical      |
 * | `number`     | Hexadecimal       |
 * | `Date`       | Chronological     |
 *
 * @param order The order to sort in. Possible values are 'descending' (largest first)
 * and 'ascending' (smallest first). Default is 'ascending'.
 */
export declare function sort<T extends Array<string | Date | number>>(array: T, order?: SortOrder): T;
/** Clears all elements from the given array and returns the array. */
export declare function clear<T extends unknown[]>(array: T): T;
/** Removes all null or undefined elements from the array and returns the array. */
export declare function clearNull<T extends unknown[]>(array: T): T;
/**
 * Out of all the numbers in the array, returns the number closest to the given target number.
 *
 * If the array is empty, returns the target number.
 */
export declare function closest(array: readonly number[], target: number): number;
/**
 * Out of all the items in the array, returns the item whose callback function returns the number
 * closest to the given target number.
 *
 * If the array is empty, returns the target number.
 */
export declare function closest<T>(array: readonly T[], callback: (item: T) => number, target: number): T;
/**
 * Moves each array item by `by`. This method will wrap the array. As a result, there will never be a negative or unfilled index.
 */
export declare function moveAll<T extends unknown[]>(array: T, by: number): T;
export declare type TransformTo1DArray<T extends unknown[]> = {
    [K in keyof T]: T[K] extends (infer U)[] ? U : T[K];
};
/**
 * Zips the selected arrays, creating a new nested array where the number of elements per level is equal to the number of passed arrays.
 * @example zip([0, 1, 2], [4, 5, 6]) => [[0, 4], [1, 5], [2, 6]]
 */
export declare function zip<T, Args extends Array<ReadonlyArray<unknown>>>(source: readonly T[], ...arrays: Args): Array<[T, ...TransformTo1DArray<Args>]>;
declare type NonArray<T extends unknown[]> = {
    [K in keyof T]: ArrayType<T[K]>;
};
/**
 * Loops through multiple arrays at once, calling the specified callback
 * function with the corresponding element at that index for each array.
 *
 * Note that all arrays must have the same size or the function fill throw.
 */
export declare function correlate<T extends unknown[][]>(...args: [...sources: T, callback: (...sources: NonArray<T>) => void]): void;
/**
 * Groups the values in the array by the return value of the property callback.
 *
 * If the return value of the `property` callback implements `Equatable`, this
 * function will call `equals()` to group the objects.
 *
 * Ordinarily the return value will be a grouped two-dimensional array. You can
 * transform the return value by passing an additional callback argument that
 * will be called once per group, for instance to return an object in the shape
 * of `{ property, items }` per group.
 *
 * Note that this callback is called during the **creation** of the group, so
 * the `items` argument will only contain one item at that point. Do not apply
 * any transformation functions like `.map()` or `.filter()` to `items`; use it
 * only to store the grouped results in a specific place, for instance by returning
 * a single-dimensional array of objects in the form of
 * `Array<{ type: GroupedPropertyValue, items: T[] }>`
 * instead of a 2-dimensional array.
 */
export declare function groupBy<T>(array: readonly T[], property: (item: T) => unknown): T[][];
export declare function groupBy<T, U, Result>(array: readonly T[], property: (item: T) => U, mapGroup: (property: U, items: readonly T[]) => Result): Result[];
/**
 * Returns a new array with all duplicate elements removed.
 *
 * Note that this function only compares values for value types,
 * otherwise it compares references.
 *
 * Prefer using
 * [Set](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Set)
 * instead of this function to avoid unnecessary memory allocation.
 */
export declare function distinct<T>(iterable: Iterable<T>): T[];
/**
 * Returns `true` if the array contains duplicate values.
 *
 * Note that this function only compares values for value types,
 * otherwise it compares references.
 */
export declare function hasDuplicates(iterable: Iterable<unknown>): boolean;
/**
 * Returns `true` if the array contains multiple of the specified value.
 *
 * Note that this function only compares values for value types,
 * otherwise it compares references.
 */
export declare function hasMultiple<T>(array: readonly T[], value: T): boolean;
/**
 * Generates an array of numbers between the specified boundaries.
 * Note that both boundaries are inclusive, meaning they will be included
 * in the range as well.
 *
 * Note that if the step size or the boundaries are floating point numbers,
 * the array will be filled until the next number would be greater than `to`,
 * regardless of whether or not `to` itself is included in the array.
 *
 * @param step The step between each number in the generated range. `1` by default.
 */
export declare function range(from: number, to: number, step?: number): number[];
/**
 * Returns the indices for all objects that match the predicate in the array.
 * This function is parallel to `Array.prototype.findIndex()`, but whereas `findIndex()`
 * only returns the first matching index, this function returns all of them.
 */
export declare function findIndices<T>(array: T[], predicate: (object: T) => boolean): number[];
/**
 * Returns all indices for the matching object in the array.
 * This function is parallel to `Array.prototype.indexOf()`, but whereas `indexOf()`
 * only returns the first matching index, this function returns all of them.
 */
export declare function findIndices<T>(array: T[], object: T): number[];
/**
 * Creates a new array with the elements from all the given arrays.
 * This function differs from `Array.prototype.concat()` in how it handles
 * duplicates: `concat()` will simply concatenate two arrays regardless of
 * duplication. This function will only add an element if it was not already
 * added by one of the other arrays.
 */
export declare function union<T>(...arrays: readonly T[][]): T[];
/**
 * Creates a new array with only the elements common to all the given arrays.
 */
export declare function intersection<T>(...arrays: readonly T[][]): T[];
/**
 * Creates a new array with only the elements that are unique to one of the
 * given arrays. In other words: the resulting array will contain all elements
 * except those shared by multiple of the given arrays.
 */
export declare function difference<T>(...arrays: readonly T[][]): T[];
/**
 * Maps an iterable of `T` to an array of `U` by
 * applying a callback to each element in turn.
 *
 * If the callback returns `null` or `undefined` for any element, the element
 * is discarded.
 *
 * Combines
 * [`Array.prototype.filter`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/filter)
 * and
 * [`Array.prototype.map`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/map).
 */
export declare function filterMap<T, U, C extends Iterable<T>>(array: C, mapFn: (object: T) => Nullable<U>): U[];
export {};
