/**
 * @file Offers convenience methods on arrays.
 *
 * Note that unknown iterations use `Array.prototype.forEach()` and Co.
 * In older browsers, a simple `for` loop was significantly faster.
 * In modern browsers, that is still the case, but the difference
 * is now negligible. Even on arrays with a length of over 10,000,000,
 * the difference in performance makes up barely 30 ms.
 */
import * as Objects from '../objects/index.js';
/**
 * Returns a copy of the specified array.
 * Note that only the array is copied, not the elements within.
 */
export function clone(array) {
    return array.slice();
}
/**
 * Returns the number of elements in the iterable or array-like.
 *
 * If the iterable contains a `length` property (such as in {@link Array})
 * or a `size` property (such as in {@link Set}), this function will use
 * them to obtain the number of elements in the iterable. Otherwise it
 * will count the number of elements by iterating over the iterable manually.
 */
export function count(iterable) {
    if (Objects.hasProperty(iterable, 'length', 'number')) {
        return iterable.length;
    }
    if (Objects.hasProperty(iterable, 'size', 'number')) {
        return iterable.size;
    }
    let count = 0;
    for (const _ of iterable) {
        count++;
    }
    return count;
}
/**
 * Gets the first element in the iterable,
 * or undefined if the iterable contains no elements.
 */
export function first(iterable) {
    if (Array.isArray(iterable)) {
        return iterable[0];
    }
    for (const element of iterable) {
        return element;
    }
    return undefined;
}
/**
 * Gets the index of the last element in the iterable,
 * or `-1` if the iterable contains no elements.
 */
export function lastIndex(iterable) {
    if (Array.isArray(iterable)) {
        return iterable.length - 1;
    }
    let i = -1;
    for (const _ of iterable) {
        i++;
    }
    return i;
}
/**
 * Gets the last element in the iterable,
 * or undefined if the iterable contains no elements.
 */
export function last(iterable) {
    if (Array.isArray(iterable)) {
        return iterable[iterable.length - 1];
    }
    let last = undefined;
    for (const element of iterable) {
        last = element;
    }
    return last;
}
/**
 * Returns true if the given index is in-bounds of the array, i.e. greater
 * than 0 and smaller than the array's length.
 *
 * Note that this function does **not** check whether there is actually an
 * array element at the specified index, as in when an array is initialized
 * with empty slots (e.g. using `Array`'s length constructor).
 * Unless you have a good reason to use `isInBounds()`, you probably want to use
 * `Arrays.hasElementAt()` instead.
 */
export function isInBounds(array, index) {
    return index >= 0 && index < array.length;
}
/**
 * Returns true if the given index refers to an actual element on the specified
 * array.
 *
 * @example
 * const array = new Array(5);     // contains 5 empty "slots"
 * Arrays.isInBounds(array, 2);    // -> true
 * Arrays.hasElementAt(array, 2);  // -> false
 */
export function hasElementAt(array, index) {
    return index in array;
}
/**
 * Gets the index of the next element in the array, starting at the given index.
 * If the index belongs to the last element in the array, returns the index of
 * the first element. Returns `-1` if the given index is out of bounds.
 */
export function nextIndex(array, fromIndex) {
    if (!isInBounds(array, fromIndex)) {
        return -1;
    }
    if (fromIndex === array.length - 1) {
        return 0;
    }
    return fromIndex + 1;
}
/**
 * Gets the next element in the array, starting at the given index. If the index belongs to the
 * last element in the array, returns the first element. Returns undefined if the given index
 * is out of bounds.
 */
export function next(array, fromIndex) {
    return array[nextIndex(array, fromIndex)];
}
/**
 * Gets the index of the previous element in the array, starting at the given
 * index. If the index belongs to the first element in the array, returns the
 * index of the last element. Returns `-1` if the given index is out of bounds.
 */
export function previousIndex(array, fromIndex) {
    if (!isInBounds(array, fromIndex)) {
        return -1;
    }
    if (fromIndex === 0) {
        return array.length - 1;
    }
    return fromIndex - 1;
}
/**
 * Gets the previous element in the array, starting at the given index. If the index belongs to the
 * first element in the array, returns the last element. Returns undefined if the given index
 * is out of bounds.
 */
export function previous(array, fromIndex) {
    return array[previousIndex(array, fromIndex)];
}
/** Returns true if the given array is empty, otherwise false. */
export function isEmpty(iterable) {
    if (Objects.hasProperty(iterable, 'length', 'number')) {
        return iterable.length === 0;
    }
    if (Objects.hasProperty(iterable, 'size', 'number')) {
        return iterable.size === 0;
    }
    for (const _ of iterable) {
        return false;
    }
    return true;
}
/** Returns true if the given array is not empty, otherwise false. */
export function isNotEmpty(iterable) {
    return !isEmpty(iterable);
}
/**
 * Compares the contents of two arrays for referential equality and returns true
 * if both arrays have the same elements in the same positions.
 */
export function equal(array1, array2) {
    if (array1.length !== array2.length) {
        return false;
    }
    for (let i = 0; i < array1.length; i++) {
        if (array1[i] !== array2[i]) {
            return false;
        }
    }
    return true;
}
/**
 * Removes the specified element(s) from the array and returns the array.
 *
 * If the array contains multiples of the specified element(s), all of them
 * are removed.
 *
 * If the array does not contain one of the elements, throws an error.
 */
export function remove(array, ...elements) {
    for (const element of elements) {
        let index = array.indexOf(element);
        if (index === -1) {
            throw new Error('Arrays.remove(): element not found in array!');
        }
        while (index !== -1) {
            array.splice(index, 1);
            index = array.indexOf(element);
        }
    }
    return array;
}
/**
 * Removes the element at the specified index from the array
 * and returns the array.
 *
 * @param index The zero-based index of the element to be removed.
 *              If negative, iterates backwards from the end of the array.
 */
export function removeAt(array, index) {
    if (!hasElementAt(array, index)) {
        throw new Error(`Failed to remove element: no element found at index ${index}.`);
    }
    array.splice(index, 1);
    return array;
}
/**
 * Replaces the specified element with another and returns the array.
 *
 * If the array contains multiples of the target element, it will only replace the first occurrence.
 */
export function replace(array, element, replacement) {
    const index = array.indexOf(element);
    if (index === -1) {
        throw new Error('Arrays.replace(): element not found in array!');
    }
    array.splice(index, 1, replacement);
    return array;
}
/**
 * Replaces all elements in the array with all elements from another array.
 */
export function replaceAll(array, replacement) {
    array.length = 0;
    array.push(...replacement);
    return array;
}
/**
 * Inserts one or multiple elements at the given index.
 *
 * This function throws an error if the index is not in-bounds.
 */
export function insert(array, index, ...elements) {
    if (!isInBounds(array, index)) {
        throw new Error(`Index ${index} is not in-bounds of the array.`);
    }
    array.splice(index, 0, ...elements);
    return array;
}
export function type(array, type) {
    if (typeof type === 'string') {
        return array.every(item => typeof item === type);
    }
    return array.every(item => item instanceof type);
}
export function sort(array, orderOrSortFn = 'ascending', ...sortFns) {
    if (array.length <= 1) {
        // No sort necessary/possible.
        return array;
    }
    if (typeof orderOrSortFn === 'string') {
        const order = orderOrSortFn;
        if (type(array, 'number')) {
            return array.sort((a, b) => order === 'descending' ? b - a : a - b);
        }
        if (type(array, 'string')) {
            array.sort();
            if (order === 'descending') {
                array.reverse();
            }
            return array;
        }
        if (type(array, Date)) {
            return array.sort((a, b) => order === 'descending' ? b.getTime() - a.getTime() : a.getTime() - b.getTime());
        }
        throw new Error('Arrays.sort(): Can\'t use a default sort() on an array that is not composed of numbers, strings, or Dates. Please use Array.prototype.sort() for other data types or use Arrays.sort() with one or more sort functions.');
    }
    const sortFunctions = [orderOrSortFn, ...sortFns];
    return array.sort((a, b) => {
        for (const sortFunction of sortFunctions) {
            const result = sortFunction(a, b);
            if (result !== 0) {
                return result;
            }
        }
        return 0;
    });
}
/** Clears all elements from the given array and returns the array. */
export function clear(array) {
    array.length = 0;
    return array;
}
/** Removes all null or undefined elements from the array and returns the array. */
export function clearNull(array) {
    let i = 0;
    while (i < array.length) {
        if (array[i] == null) {
            array.splice(i, 1);
        }
        else {
            i++;
        }
    }
    return array;
}
export function closest(array, callbackOrTarget, target) {
    const usesCallback = typeof callbackOrTarget === 'function';
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const referenceNumber = usesCallback ? target : callbackOrTarget;
    if (array.length === 0) {
        return referenceNumber;
    }
    if (usesCallback) {
        const callback = callbackOrTarget;
        return clone(array).sort((a, b) => Math.abs(referenceNumber - callback(a)) - Math.abs(referenceNumber - callback(b)))[0];
    }
    if (!type(array, 'number')) {
        throw new Error('If the array is not an array of numbers, a callback must be specified that returns a number for each item in the array.');
    }
    return clone(array).sort((a, b) => Math.abs(referenceNumber - a) - Math.abs(referenceNumber - b))[0];
}
/**
 * Moves each array item by `by`. This method will wrap the array. As a result, there will never be a negative or unfilled index.
 */
export function moveAll(array, by) {
    by = by % array.length;
    if (by === 0 || by === array.length) {
        return array;
    }
    else if (by > 0) {
        array.push(...array.splice(0, array.length - by));
        return array;
    }
    else {
        array.push(...array.splice(0, Math.abs(by)));
        return array;
    }
}
/**
 * Zips the selected arrays, creating a new nested array where the number of elements per level is equal to the number of passed arrays.
 * @example zip([0, 1, 2], [4, 5, 6]) => [[0, 4], [1, 5], [2, 6]]
 */
export function zip(source, ...arrays) {
    if (arrays.some(x => x.length !== source.length)) {
        throw new Error(`Arrays are not of identical length! Expected length ${source.length}, but found ${arrays.map(x => x.length).join(', ')}.`);
    }
    return source.map((x, i) => [x, ...arrays.map(array => array[i])]);
}
/**
 * Loops through multiple arrays at once, calling the specified callback
 * function with the corresponding element at that index for each array.
 *
 * Note that all arrays must have the same size or the function fill throw.
 */
export function correlate(...args) {
    const callback = last(args);
    const arrays = args.slice(0, -1);
    if (typeof callback !== 'function') {
        throw new Error('Arrays.correlate(): last argument must be a callback function.');
    }
    if (arrays.length <= 1) {
        throw new Error('Arrays.correlate(): must specify at least two arrays.');
    }
    if (!arrays.every(array => array.length === arrays[0].length)) {
        throw new Error('Arrays.correlate(): all arrays must be of the same length.');
    }
    for (let i = 0; i < arrays[0].length; i++) {
        const callbackArguments = arrays.map(array => array[i]);
        callback(...callbackArguments);
    }
}
export function groupBy(array, property, mapGroup = (_, items) => items) {
    const properties = [];
    const groups = [];
    const arrays = [];
    for (const item of array) {
        const result = property(item);
        const index = properties.findIndex(property => Objects.equal(result, property));
        if (index === -1) {
            const array = [item];
            const group = mapGroup(result, array);
            properties.push(result);
            groups.push(group);
            arrays.push(array);
        }
        else {
            arrays[index].push(item);
        }
    }
    return groups;
}
/**
 * Returns a new array with all duplicate elements removed.
 *
 * Note that this function only compares values for value types,
 * otherwise it compares references.
 *
 * Prefer using
 * [Set](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Set)
 * instead of this function to avoid unnecessary memory allocation.
 */
export function distinct(iterable) {
    return Array.from(new Set(iterable));
}
/**
 * Returns `true` if the array contains duplicate values.
 *
 * Note that this function only compares values for value types,
 * otherwise it compares references.
 */
export function hasDuplicates(iterable) {
    return new Set(iterable).size !== count(iterable);
}
/**
 * Returns `true` if the array contains multiple of the specified value.
 *
 * Note that this function only compares values for value types,
 * otherwise it compares references.
 */
export function hasMultiple(array, value) {
    return array.indexOf(value) !== array.lastIndexOf(value);
}
/**
 * Generates an array of numbers between the specified boundaries.
 * Note that both boundaries are inclusive, meaning they will be included
 * in the range as well.
 *
 * Note that if the step size or the boundaries are floating point numbers,
 * the array will be filled until the next number would be greater than `to`,
 * regardless of whether or not `to` itself is included in the array.
 *
 * @param step The step between each number in the generated range. `1` by default.
 */
export function range(from, to, step = 1) {
    if (step === 0) {
        throw new Error('Arrays.range(): step must not be 0.');
    }
    const inverted = to < from;
    const normalizedStep = Math.abs(step);
    let length = Math.abs(to - from) / normalizedStep;
    length = Math.trunc(length) === length ? length + 1 : Math.ceil(length);
    const array = new Array(length);
    for (let i = 0; i < length; i++) {
        array[i] = from + (inverted ? -i : i) * normalizedStep;
    }
    return array;
}
export function findIndices(array, objectOrPredicate) {
    const predicate = typeof objectOrPredicate === 'function' && objectOrPredicate.length === 1
        ? objectOrPredicate
        : (object) => object === objectOrPredicate;
    return array.reduce((acc, value, index) => {
        if (predicate(value)) {
            return acc.concat(index);
        }
        return acc;
    }, []);
}
/**
 * Creates a new array with the elements from all the given arrays.
 * This function differs from `Array.prototype.concat()` in how it handles
 * duplicates: `concat()` will simply concatenate two arrays regardless of
 * duplication. This function will only add an element if it was not already
 * added by one of the other arrays.
 */
export function union(...arrays) {
    const result = arrays.length === 0 ? [] : arrays[0].slice();
    for (let i = 1; i < arrays.length; i++) {
        for (const element of arrays[i]) {
            if (!result.includes(element)) {
                result.push(element);
            }
        }
    }
    return result;
}
/**
 * Creates a new array with only the elements common to all the given arrays.
 */
export function intersection(...arrays) {
    if (arrays.length === 1) {
        return arrays[0].slice();
    }
    const result = [];
    if (arrays.length === 0) {
        return result;
    }
    for (const element of arrays[0]) {
        let shouldAdd = true;
        for (let i = 1; i < arrays.length; i++) {
            if (!arrays[i].includes(element)) {
                shouldAdd = false;
                break;
            }
        }
        if (shouldAdd) {
            result.push(element);
        }
    }
    return result;
}
/**
 * Creates a new array with only the elements that are unique to one of the
 * given arrays. In other words: the resulting array will contain all elements
 * except those shared by multiple of the given arrays.
 */
export function difference(...arrays) {
    if (arrays.length === 1) {
        return arrays[0].slice();
    }
    if (arrays.length === 0) {
        return [];
    }
    return arrays.flat(1)
        .filter(element => {
        let count = 0;
        for (const array of arrays) {
            if (array.includes(element)) {
                count++;
            }
        }
        return count === 1;
    });
}
/**
 * Maps an iterable of `T` to an array of `U` by
 * applying a callback to each element in turn.
 *
 * If the callback returns `null` or `undefined` for any element, the element
 * is discarded.
 *
 * Combines
 * [`Array.prototype.filter`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/filter)
 * and
 * [`Array.prototype.map`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/map).
 */
export function filterMap(array, mapFn) {
    const newElements = Array.isArray(array) ? new array.constructor() : [];
    for (const element of array) {
        const result = mapFn(element);
        if (result != null) {
            newElements.push(result);
        }
    }
    return newElements;
}
