/**
 * @file Offers assertion functions on a variety of data types.
 *
 * Assertions are boolean conditionals that should always return true.
 * If they return false, it is indicative of a logic error, not a user error.
 * They are commonly used to make sure an argument is in the valid range of values.
 *
 * By default, assertion errors are prefixed with "Assertion failed" to distinguish
 * them from other errors. To disable this behaviour, set `Assert.prefixed = false`.
 *
 * All assertions inherit from a custom `AssertionError`. This allows you to check
 * if an error comes from an assertion, for instance like so:
 *
 * @example
 *
 * try {
 *      Assert.that(false, 'Assertion error');
 *
 *      throw new Error('Another error.');
 * } catch (e) {
 *      if (e.name === 'AssertionError') {
 *          // ...
 *      } else {
 *          // ...
 *      }
 * }
 */
import { StringifyOptions } from '../objects/index.js';
import { TypeofResult, Typeof, Constructor, Falsy, Truthy } from '../types.js';
export declare class AssertionError extends Error {
    constructor(message?: string);
}
interface AssertConfiguration {
    /**
     * If true, prefixes all assertion error messages with
     * "Assertion failed." to distinguish them from other errors.
     */
    prefixed?: boolean;
    /**
     * Determines options for how values should be stringified in assertion
     * failure messages.
     *
     * By default array and object contents are truncated. All other options
     * are left at their default.
     */
    stringifyOptions?: StringifyOptions;
}
/** Configures the assert module with the given options. */
export declare function configure(configuration: AssertConfiguration): void;
export declare let prefixed: boolean;
export declare let stringifyOptions: StringifyOptions;
/**
 * Throws a generic assertion error if the condition is not true and
 * optionally supplies the error with the given failure message.
 *
 * It is recommended to use one of the other assertion types for a more
 * expressive error message.
 */
export declare function that(condition: boolean, failureMessage?: string): asserts condition;
/**
 * Throws an assertion error if the value does not equal the specified value.
 * This function compares by referential equality.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function equals<T>(actual: unknown, expected: T, name?: string): asserts actual is T;
/**
 * Throws an assertion error if the value equals the specified value.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function notEquals(actual: unknown, unexpected: unknown, name?: string): void;
/**
 * Throws an assertion error if the value is not truthy.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function truthy<T>(value: T, name?: string): asserts value is Truthy<T>;
/**
 * Throws an assertion error if the value is not falsy.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function falsy<T>(value: T, name?: string): asserts value is Falsy<T>;
/**
 * Throws an assertion error if the value is null or undefined.
 * Useful for checking the existence of mandatory arguments.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function notNull<T>(value: T, name?: string): asserts value is NonNullable<T>;
/**
 * Throws an assertion error if the type of the value is not the specified type.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function type<T extends TypeofResult>(value: unknown, type: T, name?: string): asserts value is Typeof<T>;
/**
 * Throws an assertion error if the type of the value is not the specified type.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function instanceOf<T>(value: unknown, constructor: Constructor<T>, name?: string): asserts value is T;
/**
 * Throws an assertion error if the iterable contains elements.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function empty(iterable: Iterable<unknown>, name?: string): void;
/**
 * Throws an assertion error if the iterable contains no elements.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function notEmpty(iterable: Iterable<unknown>, name?: string): void;
/**
 * Iterates across the given array and calls the specified callback on each
 * array item. If any of the callbacks throw an assertion error, stops iterating
 * and propagates the assertion error with an expressive failure message
 * that shows which element failed the assertion.
 *
 * Note that, while returning a simple boolean (instead of using an assertion
 * function inside the callback) is not encouraged, it does generate an
 * appropriate assertion error.
 *
 * @param name If you're checking a named value (like a variable or property),
 *   you can enter its name here for a more expressive error message.
 */
export declare function every<T>(array: readonly T[], callback: (value: T, index: number) => void | boolean, name?: string): void;
export {};
