/**
 * Represents a queue, i.e. a collection of elements structured in a
 * first-in-first-out (FIFO) manner. New elements are added to the *back*
 * of the queue, but elements are always retrieved from the *front*
 * of the queue.
 *
 * ## Internals and performance
 *
 * This type is implemented using a
 * [circular buffer](https://en.wikipedia.org/wiki/Circular_buffer)
 * backed by an array.
 * This makes the performance on most operations O(1). The only exception
 * is when a new element is to be added while the internal buffer is already
 * at capacity, in which case the buffer must first be resized.
 * Resizing does carry some performance penalty with it but is generally fast
 * enough not to be a concern for most code. If it is, however, you may use
 * {@link Queue.withCapacity}() to construct a Queue with a specified base
 * capacity, eliminating the need for resizing.
 */
export default class Queue<T> implements Iterable<T> {
    #private;
    private readonly array;
    private start;
    private end;
    /**
     * Constructs a new Queue.
     *
     * The initial size of the Queue will always be larger than the number
     * of elements added here during construction. If no elements are added,
     * the initial size of the Queue will be `24`. To instantiate a Queue
     * with a bigger base size, use {@link Queue.withCapacity}().
     *
     * @param elements Initial elements to add to the Queue.
     */
    constructor(...elements: T[]);
    [Symbol.iterator](): Iterator<T>;
    /**
     * Constructs a new Queue with the given capacity. Useful for queues with a
     * small initial size that may accept a great number of new elements.
     */
    static withCapacity<T>(capacity: number): Queue<T>;
    /**
     * Adds an element to the back of the Queue. This is an O(1) operation,
     * unless the queue is at capacity, in which case it has to be resized
     * first.
     */
    enqueue(element: T): this;
    /**
     * Removes the frontmost element from the Queue and returns it.
     *
     * This function will throw an error if the Queue is empty.
     */
    dequeue(): T;
    /**
     * Returns the frontmost element without removing it from the Queue.
     * If the Queue is empty, returns `undefined`.
     */
    peek(): T | undefined;
    /** Gets the number of elements in this Queue. */
    get size(): number;
    /** Returns `true` if the Queue is empty, otherwise `false`. */
    isEmpty(): boolean;
    /** Returns an array representation of this Queue. */
    toJSON(): T[];
}
