// TODO:
/* v8 ignore start */
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Queue_instances, _Queue_size, _Queue_expand;
import { nextIndex, previousIndex } from '../arrays/index.js';
const Empty = Symbol('empty-queue-element');
const QUEUE_BUFFER_SIZE = 24;
/**
 * Represents a queue, i.e. a collection of elements structured in a
 * first-in-first-out (FIFO) manner. New elements are added to the *back*
 * of the queue, but elements are always retrieved from the *front*
 * of the queue.
 *
 * ## Internals and performance
 *
 * This type is implemented using a
 * [circular buffer](https://en.wikipedia.org/wiki/Circular_buffer)
 * backed by an array.
 * This makes the performance on most operations O(1). The only exception
 * is when a new element is to be added while the internal buffer is already
 * at capacity, in which case the buffer must first be resized.
 * Resizing does carry some performance penalty with it but is generally fast
 * enough not to be a concern for most code. If it is, however, you may use
 * {@link Queue.withCapacity}() to construct a Queue with a specified base
 * capacity, eliminating the need for resizing.
 */
export default class Queue {
    /**
     * Constructs a new Queue.
     *
     * The initial size of the Queue will always be larger than the number
     * of elements added here during construction. If no elements are added,
     * the initial size of the Queue will be `24`. To instantiate a Queue
     * with a bigger base size, use {@link Queue.withCapacity}().
     *
     * @param elements Initial elements to add to the Queue.
     */
    constructor(...elements) {
        _Queue_instances.add(this);
        this.start = 0;
        _Queue_size.set(this, 0);
        if (elements.length === 1 && elements[0] === Empty) {
            // Small hack to make it possible for internals to construct
            // a Queue without expanding it immediately.
            this.array = [];
            this.end = 0;
            return;
        }
        this.array = elements;
        __classPrivateFieldSet(this, _Queue_size, elements.length, "f");
        this.end = elements.length - 1;
        __classPrivateFieldGet(this, _Queue_instances, "m", _Queue_expand).call(this);
    }
    *[(_Queue_size = new WeakMap(), _Queue_instances = new WeakSet(), Symbol.iterator)]() {
        for (let i = this.start; i < this.end; i = nextIndex(this.array, i)) {
            yield this.array[i];
        }
    }
    /**
     * Constructs a new Queue with the given capacity. Useful for queues with a
     * small initial size that may accept a great number of new elements.
     */
    static withCapacity(capacity) {
        const queue = new Queue(Empty);
        __classPrivateFieldGet(queue, _Queue_instances, "m", _Queue_expand).call(queue, capacity);
        return queue;
    }
    /**
     * Adds an element to the back of the Queue. This is an O(1) operation,
     * unless the queue is at capacity, in which case it has to be resized
     * first.
     */
    enqueue(element) {
        var _a;
        const previousEnd = previousIndex(this.array, this.end);
        if (this.start === previousEnd) {
            __classPrivateFieldGet(this, _Queue_instances, "m", _Queue_expand).call(this);
        }
        this.start = previousIndex(this.array, this.start);
        this.array[this.start] = element;
        __classPrivateFieldSet(this, _Queue_size, (_a = __classPrivateFieldGet(this, _Queue_size, "f"), _a++, _a), "f");
        return this;
    }
    /**
     * Removes the frontmost element from the Queue and returns it.
     *
     * This function will throw an error if the Queue is empty.
     */
    dequeue() {
        var _a;
        if (this.isEmpty()) {
            throw new Error('Can\'t dequeue(): Queue is empty.');
        }
        const previousEnd = previousIndex(this.array, this.end);
        const head = this.array[this.end];
        this.array[this.end] = Empty;
        this.end = previousEnd;
        __classPrivateFieldSet(this, _Queue_size, (_a = __classPrivateFieldGet(this, _Queue_size, "f"), _a--, _a), "f");
        return head;
    }
    /**
     * Returns the frontmost element without removing it from the Queue.
     * If the Queue is empty, returns `undefined`.
     */
    peek() {
        if (this.isEmpty()) {
            return undefined;
        }
        return this.array[this.end];
    }
    /** Gets the number of elements in this Queue. */
    get size() {
        return __classPrivateFieldGet(this, _Queue_size, "f");
    }
    /** Returns `true` if the Queue is empty, otherwise `false`. */
    isEmpty() {
        return this.start === this.end;
    }
    /** Returns an array representation of this Queue. */
    toJSON() {
        return [...this];
    }
}
_Queue_expand = function _Queue_expand(to) {
    const previousLength = this.array.length;
    const expandBy = to != null
        ? this.array.length - to
        : this.array.length < QUEUE_BUFFER_SIZE
            ? QUEUE_BUFFER_SIZE
            : this.array.length;
    for (let i = 0; i < expandBy; i++) {
        this.array.push(Empty);
    }
    if (this.start > this.end) {
        // Move all values at the end of the array up to the new end.
        const lengthDifference = previousLength - this.start;
        for (let i = this.start; i < previousLength; i++) {
            this.array[i + lengthDifference] = this.array[i];
            this.array[i] = Empty;
        }
        this.start += lengthDifference;
    }
};
