// TODO:
/* v8 ignore start */
/**
 * Represents a classic stack-based type. A stack is last-in-first-out (LIFO),
 * so you can only ever add or remove elements from the top of the stack
 * (the most recently added element).
 *
 * To ensure the stack uses contiguous memory, this data type uses an
 * array internally.
 */
export default class Stack {
    /**
     * Constructs a new Stack.
     *
     * @param elements Initial elements to add to the Stack.
     *   Note that these are added in-sequence, i.e. the first element
     *   is the one at the *bottom*, not the top.
     */
    constructor(...elements) {
        this.array = elements;
    }
    [Symbol.iterator]() {
        return this.array[Symbol.iterator]();
    }
    /**
     * Adds an element to the top of the Stack.
     */
    push(element) {
        this.array.push(element);
    }
    /**
     * Removes the topmost element from the stack and returns it.
     *
     * This function will throw an error if the Stack is empty.
     */
    pop() {
        const last = this.array.pop();
        if (last == null) {
            throw new Error(`Can't pop(): Stack had no elements.`);
        }
        return last;
    }
    /** Gets the number of elements in the Stack. */
    get size() {
        return this.array.length;
    }
    /**
     * Returns the topmost element without removing it from the Stack.
     * If the Stack is empty, returns `undefined`.
     */
    peek() {
        return this.array[this.array.length - 1];
    }
    /** Returns true if the Stack is empty. */
    isEmpty() {
        return this.array.length === 0;
    }
    /** Returns an array representation of this Stack. */
    toJSON() {
        return [...this];
    }
}
