function throwOnInvalidFlag(value) {
    if (value < 0 || !Number.isInteger(value) || Number.isNaN(value) || !Number.isFinite(value)) {
        throw new Error(`Invalid flag value. Flags must be positive integers.`);
    }
}
/**
 * A convenience class allowing bitwise operations on a flags value.
 * This class also contains static methods that can be used without
 * constructing an instance.
 */
export default class Flags {
    constructor(initialValue) {
        if (initialValue != null && initialValue !== 0) {
            throwOnInvalidFlag(initialValue);
        }
        this.value = initialValue !== null && initialValue !== void 0 ? initialValue : 0;
    }
    /** Checks if the number includes the given flag. */
    has(flag) {
        return Flags.has(this.value, flag);
    }
    /** Checks if the number includes any of the given flags. */
    hasSome(...flags) {
        return Flags.hasSome(this.value, ...flags);
    }
    /** Checks if the number includes all of the given flags. */
    hasEvery(...flags) {
        return Flags.hasEvery(this.value, ...flags);
    }
    /** Sets the given flags. */
    set(...flags) {
        this.value = Flags.set(this.value, ...flags);
        return this;
    }
    /** Unsets the given flags. */
    unset(...flags) {
        this.value = Flags.unset(this.value, ...flags);
        return this;
    }
    /** Clears all flags. */
    clear() {
        this.value = 0;
        return this;
    }
    /** Checks if the number includes the given flag. */
    static has(flags, flag) {
        throwOnInvalidFlag(flag);
        if (flags === 0) {
            return false;
        }
        return flag === (flags & flag);
    }
    /** Checks if the number includes any of the given flags. */
    static hasSome(flags, ...someFlags) {
        return someFlags.some(flag => flag === 0 || Flags.has(flags, flag));
    }
    /** Checks if the number includes all of the given flags. */
    static hasEvery(flags, ...everyFlag) {
        return everyFlag.every(flag => Flags.has(flags, flag));
    }
    /** Returns a new number with the given flags set on `flags`. */
    static set(flags, ...flagsToSet) {
        for (const flag of flagsToSet) {
            throwOnInvalidFlag(flag);
            flags |= flag;
        }
        return flags;
    }
    /** Returns a new number with the given flags unset on `flags`. */
    static unset(flags, ...flagsToUnset) {
        for (const flag of flagsToUnset) {
            throwOnInvalidFlag(flag);
            flags = flags & ~flag;
        }
        return flags;
    }
    /** Converts the flags to a number. */
    toNumber() {
        return this.value;
    }
    /** Converts the flags to a number. */
    toJSON() {
        return this.value;
    }
    /** Converts the flags to a number string. */
    toString() {
        return this.value.toString(10);
    }
}
