import type { TypeofResult, Constructor, Predicate } from '../types.js';
declare type ReturnType<T> = boolean | ((...objects: T[]) => boolean);
interface FluentIs<T, U extends ReturnType<T> = boolean> {
    /**
     * By default, all objects passed into `is()` must meet the condition.
     * By using this method, the condition will instead return `true`
     * if any objects pass the check.
     */
    any(): FluentIs<T, U>;
    /**
     * If `any()` was previously called in this chain, resets the chain
     * back to its default behaviour of every object having to meet the
     * specified condition.
     * If `any()` was not called, this function has no effect and can
     * be used solely for more expressive syntax.
     */
    every(): FluentIs<T, U>;
    /** Inverts the condition of this `is()` expression. */
    get not(): FluentIs<T, U>;
    /**
     * Returns `true` if and only if all passed matchers evaluate to `true`.
     */
    matching(...matchers: Array<Predicate<T>>): U;
    /**
     * Returns `true` if any of the passed matchers evaluate to `true`.
     */
    matchingSome(...matchers: Array<Predicate<T>>): U;
    /** @see {@link oneOf} */
    oneOf(...these: T[]): U;
    /** @see {@link truthy} */
    truthy(): U;
    /** @see {@link falsy} */
    falsy(): U;
    /** @see {@link keyOf} */
    keyOf<T extends object>(object: T): U;
    /** @see {@link arrayOf} */
    arrayOf<T>(object: T): U;
    /** @see {@link type} */
    type<T extends TypeofResult>(type: T): U;
    /** @see {@link instanceOf} */
    instanceOf<T extends Constructor>(constructor: T): U;
    /** @see {@link value} */
    value<T>(value: T): U;
    /** @see {@link notNull} */
    notNull(): U;
    /** @see {@link nullish} */
    nullish(): U;
    /** @see {@link integer} */
    integer(): U;
    /** @see {@link float} */
    float(): U;
    /** @see {@link approximately} */
    approximately(comparedValue: number, tolerance?: number): U;
    /** @see {@link array} */
    array(): U;
    /** @see {@link func} */
    func(argumentCount?: number): U;
}
export declare type EagerFluentIs<T> = FluentIs<T, boolean>;
export declare type LazyFluentIs<T> = FluentIs<T, (...objects: T[]) => boolean>;
/** @internal */
export declare const fluentIs: EagerFluentIs<unknown>;
/** @internal */
export declare const fluentIsLazy: LazyFluentIs<unknown>;
/** @internal */
export declare function set(objs: unknown[]): void;
export {};
