import { compare, isFloat } from '../numbers/index.js';
/** Matches a truthy value. */
export function truthy() {
    return (item) => new Boolean(item).valueOf();
}
/** Matches a falsy value. */
export function falsy() {
    return (item) => !item;
}
/** Matches if any of the matchers match. */
export function oneOf(...matchers) {
    return (item) => matchers.some(matcher => matcher(item));
}
/** Matches if the passed object contains the value as a key. */
export function keyOf(object) {
    return (item) => Object.keys(object).includes(`${item}`);
}
/** Matches a value of the specified type. */
export function type(type) {
    return (item) => typeof item === type;
}
/** Matches a value of the specified instance type or any inherited types. */
export function instanceOf(constructor) {
    return (item) => item instanceof constructor;
}
/** Matches a value by referential equality. */
export function value(value) {
    return (item) => item === value;
}
/** Matches a value that is not null or undefined. */
export function notNull() {
    return (item) => item != null;
}
/** Matches a value that is null or undefined. */
export function nullish() {
    return (item) => item == null;
}
/** Matches a string. */
export function string() {
    return (item) => typeof item === 'string';
}
/** Matches a number. */
export function number() {
    return (item) => typeof item === 'number';
}
/** Matches an integral number. */
export function integer() {
    return (item) => Number.isInteger(item);
}
/** Matches a floating point number (but not integral numbers). */
export function float() {
    return (item) => isFloat(item);
}
/** Matches a number that approximately equals the passed number. */
export function approximately(comparedValue, tolerance = 0.000001) {
    return (item) => typeof item === 'number' && compare(item, comparedValue, tolerance);
}
/** Matches an array. */
export function array() {
    return (item) => Array.isArray(item);
}
/** Matches an array where each element matches the given matcher. */
export function arrayOf(matcher) {
    return (item) => Array.isArray(item) && item.every(matcher);
}
export function func(argumentCount) {
    return (item) => typeof item === 'function' && (argumentCount == null || item.length === argumentCount);
}
/** Matches if all passed matchers match. */
export function every(...matchers) {
    return item => matchers.every(matcher => matcher(item));
}
/** Matches if any of the passed matchers match. */
export function any(...matchers) {
    return item => matchers.every(matcher => matcher(item));
}
