import { Equatable } from '../types.js';
import CompareResult from './CompareResult.js';
interface SemanticVersion {
    /**
     * The major component of the version. Increments whenever incompatible
     * API changes are introduced, with the exception of `0.x.y`, which is
     * for initial development and may see breaking changes at any time.
     */
    major: number;
    /**
     * The minor component of the version. Increments whenever backwards-
     * compatible functionality is added.
     */
    minor: number;
    /**
     * The patch component of the version. Increments whenever backwards-
     * compatible bug fixes are introduced.
     */
    patch: number;
    /**
     * The pre-release component of the version. Is separated from the other
     * components with a dash (`-`) and clearly distinguishes the version
     * from a "normal" version (e.g. an `alpha` version), marking it as
     * potentially unstable.
     */
    preRelease: string;
    /**
     * The build metadata of the version. Is separated from the other components
     * with a plus (`+`) and can contain additional information about the
     * specific build.
     */
    build: string;
}
/**
 * Represents an immutable MAJOR.MINOR.PATCH version
 * following semantic versioning rules.
 */
export default class Version implements SemanticVersion, Equatable {
    readonly major: number;
    readonly minor: number;
    readonly patch: number;
    readonly preRelease: string;
    readonly build: string;
    /**
     * Parses a version out of another version, a semantic versioning string,
     * or individual major.minor.patch components.
     *
     * An empty constructor call will result in a valid `0.0.0` version.
     */
    constructor(version: SemanticVersion | string);
    constructor(major?: number, minor?: number, patch?: number, preRelease?: string, build?: string);
    /**
     * `true` if the version is a valid semantic version,
     * `false` otherwise.
     */
    get valid(): boolean;
    /**
     * Returns a new {@link Version} with the {@link patch} component
     * incremented by 1.
     * Note that this will strip out any {@link preRelease} or {@link build}
     * labels.
     *
     * If the version is invalid, the new instance will also be invalid.
     */
    incrementPatch(): Version;
    /**
     * Returns a new {@link Version} with the {@link minor} component
     * incremented by 1.
     * Note that this will strip out any {@link preRelease} or {@link build}
     * labels and reset the {@link patch} component.
     *
     * If the version is invalid, the new instance will also be invalid.
     */
    incrementMinor(): Version;
    /**
     * Returns a new {@link Version} with the {@link major} component
     * incremented by 1.
     * Note that this will strip out any {@link preRelease} or {@link build}
     * labels and reset the {@link minor} and {@link patch} components.
     *
     * If the version is invalid, the new instance will also be invalid.
     */
    incrementMajor(): Version;
    /**
     * Compares this version with another one, returning a result based on
     * which should take precedence over the other.
     *
     * If this version is lower than the passed version in precedence, returns
     * {@link CompareResult.Less}.
     * If this version is equal to the passed version in precedence, returns
     * {@link CompareResult.Equal}.
     * If this version is greater than the passed version in precedence, returns
     * {@link CompareResult.Greater}.
     * If this version is invalid but the passed version is not, returns
     * {@link CompareResult.Less}.
     *
     * A pre-release version is always lower in precedence. If both versions
     * have a pre-release component, each dot-separated identifier is compared
     * numerically or lexically, whichever applies. If the number of identifiers
     * is not equal and all preceding identifiers are the same, the version
     * with *more* identifiers wins.
     *
     * Note that a result of `0` (or {@link CompareResult.Equal}) does **not**
     * mean both instances are identical, just that none take precedence
     * over the other. Use {@link equals}() to check for equality.
     */
    compare(version: Version | string): CompareResult;
    /** Like {@link compare}, but only compares the major component. */
    compareMajor(version: Version | string): CompareResult;
    /** Like {@link compare}, but only compares the major and minor components. */
    compareMinor(version: Version | string): CompareResult;
    /** Like {@link compare}, but only compares the major, minor, and patch components. */
    comparePatch(version: Version | string): CompareResult;
    /**
     * Returns `true` if the two versions are completely identical.
     *
     * Note that two invalid versions are always identical.
     */
    equals(version: unknown): boolean;
    /**
     * Returns the proper semantic string representation
     * of this semantic version.
     *
     * Note that invalid versions return "Invalid Version".
     */
    toJSON(): string;
    /**
     * Returns the proper semantic string representation
     * of this semantic version.
     *
     * Note that invalid versions return "Invalid Version".
     */
    toString(): string;
}
export {};
