import CompareResult from './CompareResult.js';
/** Official JS RegExp. See https://semver.org/. */
const regExp = /^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$/;
function versionString(version) {
    let string = [version.major, version.minor, version.patch]
        // Without this the string might use scientific notation
        // (which is not a valid semantic version string)
        .map(x => x.toLocaleString('fullwide', { useGrouping: false }))
        .join('.');
    if (version.preRelease.length > 0) {
        string += `-${version.preRelease}`;
    }
    if (version.build.length > 0) {
        string += `+${version.build}`;
    }
    return string;
}
function compareValidity(v1, v2) {
    if (typeof v2 === 'string') {
        v2 = new Version(v2);
    }
    if (v1.valid && v2.valid) {
        return null;
    }
    else if (!v1.valid && !v2.valid) {
        return CompareResult.Equal;
    }
    if (v2.valid) {
        return CompareResult.Less;
    }
    else {
        return CompareResult.Greater;
    }
}
/**
 * Represents an immutable MAJOR.MINOR.PATCH version
 * following semantic versioning rules.
 */
export default class Version {
    constructor(version, minor, patch, preRelease, build) {
        this.major = 0;
        this.minor = 0;
        this.patch = 0;
        this.preRelease = '';
        this.build = '';
        if (typeof version === 'string') {
            const matches = version.match(regExp);
            if (matches != null) {
                // Array will always have a length of 6.
                this.major = parseInt(matches[1], 10);
                this.minor = parseInt(matches[2], 10);
                this.patch = parseInt(matches[3], 10);
                if (matches[4] != null)
                    this.preRelease = matches[4];
                if (matches[5] != null)
                    this.build = matches[5];
            }
            else {
                this.major = this.minor = this.patch = Number.NaN;
                this.preRelease = this.build = '';
            }
        }
        else if (version == null || typeof version === 'number') {
            if (version != null)
                this.major = version;
            if (minor != null)
                this.minor = minor;
            if (patch != null)
                this.patch = patch;
            if (preRelease != null)
                this.preRelease = preRelease;
            if (build != null)
                this.build = build;
            if (!regExp.test(versionString(this))) {
                this.major = this.minor = this.patch = Number.NaN;
                this.preRelease = this.build = '';
            }
        }
        else {
            if (!regExp.test(versionString(version))) {
                this.major = this.minor = this.patch = Number.NaN;
                this.preRelease = this.build = '';
            }
            else {
                this.major = version.major;
                this.minor = version.minor;
                this.patch = version.patch;
                this.preRelease = version.preRelease;
                this.build = version.build;
            }
        }
    }
    /**
     * `true` if the version is a valid semantic version,
     * `false` otherwise.
     */
    get valid() {
        return !Number.isNaN(this.major);
    }
    /**
     * Returns a new {@link Version} with the {@link patch} component
     * incremented by 1.
     * Note that this will strip out any {@link preRelease} or {@link build}
     * labels.
     *
     * If the version is invalid, the new instance will also be invalid.
     */
    incrementPatch() {
        if (!this.valid) {
            return new Version(this);
        }
        return new Version(this.major, this.minor, this.patch + 1);
    }
    /**
     * Returns a new {@link Version} with the {@link minor} component
     * incremented by 1.
     * Note that this will strip out any {@link preRelease} or {@link build}
     * labels and reset the {@link patch} component.
     *
     * If the version is invalid, the new instance will also be invalid.
     */
    incrementMinor() {
        if (!this.valid) {
            return new Version(this);
        }
        return new Version(this.major, this.minor + 1, 0);
    }
    /**
     * Returns a new {@link Version} with the {@link major} component
     * incremented by 1.
     * Note that this will strip out any {@link preRelease} or {@link build}
     * labels and reset the {@link minor} and {@link patch} components.
     *
     * If the version is invalid, the new instance will also be invalid.
     */
    incrementMajor() {
        if (!this.valid) {
            return new Version(this);
        }
        return new Version(this.major + 1, 0, 0);
    }
    /**
     * Compares this version with another one, returning a result based on
     * which should take precedence over the other.
     *
     * If this version is lower than the passed version in precedence, returns
     * {@link CompareResult.Less}.
     * If this version is equal to the passed version in precedence, returns
     * {@link CompareResult.Equal}.
     * If this version is greater than the passed version in precedence, returns
     * {@link CompareResult.Greater}.
     * If this version is invalid but the passed version is not, returns
     * {@link CompareResult.Less}.
     *
     * A pre-release version is always lower in precedence. If both versions
     * have a pre-release component, each dot-separated identifier is compared
     * numerically or lexically, whichever applies. If the number of identifiers
     * is not equal and all preceding identifiers are the same, the version
     * with *more* identifiers wins.
     *
     * Note that a result of `0` (or {@link CompareResult.Equal}) does **not**
     * mean both instances are identical, just that none take precedence
     * over the other. Use {@link equals}() to check for equality.
     */
    compare(version) {
        if (typeof version === 'string') {
            version = new Version(version);
        }
        const patchResult = this.comparePatch(version);
        if (patchResult !== CompareResult.Equal) {
            return patchResult;
        }
        if (this.preRelease.length > 0 || version.preRelease.length > 0) {
            if (this.preRelease.length === 0) {
                return CompareResult.Greater;
            }
            else if (version.preRelease.length === 0) {
                return CompareResult.Less;
            }
            else {
                const thisIdentifiers = this.preRelease.split('.');
                const thatIdentifiers = version.preRelease.split('.');
                for (let i = 0; i < thisIdentifiers.length; i++) {
                    if (thatIdentifiers[i] == null) {
                        return CompareResult.Greater;
                    }
                    const num1 = Number(thisIdentifiers[i]);
                    const num2 = Number(thatIdentifiers[i]);
                    if (!Number.isNaN(num1) && !Number.isNaN(num2)) {
                        if (num1 === num2) {
                            continue;
                        }
                        return num1 < num2
                            ? CompareResult.Less
                            : CompareResult.Greater;
                    }
                    const result = thisIdentifiers[i].localeCompare(thatIdentifiers[i]);
                    if (result < 0) {
                        return CompareResult.Less;
                    }
                    else if (result > 0) {
                        return CompareResult.Greater;
                    }
                }
                if (thisIdentifiers.length < thatIdentifiers.length) {
                    return CompareResult.Less;
                }
            }
        }
        return CompareResult.Equal;
    }
    /** Like {@link compare}, but only compares the major component. */
    compareMajor(version) {
        if (typeof version === 'string') {
            version = new Version(version);
        }
        const validityResult = compareValidity(this, version);
        if (validityResult != null) {
            return validityResult;
        }
        if (this.major < version.major) {
            return CompareResult.Less;
        }
        else if (this.major > version.major) {
            return CompareResult.Greater;
        }
        return CompareResult.Equal;
    }
    /** Like {@link compare}, but only compares the major and minor components. */
    compareMinor(version) {
        if (typeof version === 'string') {
            version = new Version(version);
        }
        const majorResult = this.compareMajor(version);
        if (majorResult !== CompareResult.Equal) {
            return majorResult;
        }
        if (this.minor < version.minor) {
            return CompareResult.Less;
        }
        else if (this.minor > version.minor) {
            return CompareResult.Greater;
        }
        return CompareResult.Equal;
    }
    /** Like {@link compare}, but only compares the major, minor, and patch components. */
    comparePatch(version) {
        if (typeof version === 'string') {
            version = new Version(version);
        }
        const minorResult = this.compareMinor(version);
        if (minorResult !== CompareResult.Equal) {
            return minorResult;
        }
        if (this.patch < version.patch) {
            return CompareResult.Less;
        }
        else if (this.patch > version.patch) {
            return CompareResult.Greater;
        }
        return CompareResult.Equal;
    }
    /**
     * Returns `true` if the two versions are completely identical.
     *
     * Note that two invalid versions are always identical.
     */
    equals(version) {
        if (typeof version === 'object') {
            version = version === null || version === void 0 ? void 0 : version.toString();
        }
        return this.toString() === version;
    }
    /**
     * Returns the proper semantic string representation
     * of this semantic version.
     *
     * Note that invalid versions return "Invalid Version".
     */
    toJSON() {
        return this.toString();
    }
    /**
     * Returns the proper semantic string representation
     * of this semantic version.
     *
     * Note that invalid versions return "Invalid Version".
     */
    toString() {
        if (!this.valid) {
            return 'Invalid Version';
        }
        return versionString(this);
    }
}
