/**
 * @file Provides helper methods related to operations on numbers.
 *
 * Note that all range-related functions (like `clamp()`) internally use a `Range`.
 * There is no noticeable difference in performance between a function that does not
 * use a custom class, a function that instantiates a new Range every time it is called,
 * and a function that uses a cached Range every time it is called.
 */
import Range from './Range.js';
interface Configuration {
    /**
     * The default tolerance to use in functions that compare floating point numbers.
     * The default tolerance is `0.0000001`.
     */
    defaultTolerance?: number;
}
export declare function configure(configuration: Configuration): void;
/**
 * Accurately compares two integral or floating point numbers with the given tolerance.
 *
 * The tolerance determines how big the difference between the two numbers may be before
 * they are no longer considered equal. If the default tolerance has not been changed
 * via `configure()`, it is `0.0000001`.
 */
export declare function compare(value1: number, value2: number, tolerance?: number): boolean;
/**
 * Creates a new `Range` with the given `from` and `to` values.
 *
 * Ranges are powerful mathematical objects with the ability to, for instance,
 * clamp a number, check if a number is contained in a range,
 * or determine the relative percentual location of a value inside the
 * range.
 *
 * The cost of instantiating a new Range is roughly equivalent to the cost
 * of calling `Range.set()` on an existing Range, so don't be afraid of
 * calling this method. Even in tight loops, the performance impact
 * will be minimal.
 *
 * Note that `from` does not need to be smaller than `to`. Using an inverted
 * range can be helpful for, for instance, `Range.at()`.
 */
export declare function range(from: number, to: number): Range;
export declare function range(...values: number[]): Range;
/** Returns true if the number is an even number. */
export declare function isEven(value: number): boolean;
/** Returns true if the number is an odd number. */
export declare function isOdd(value: number): boolean;
/**
 * Returns true if the number is a floating point number,
 * i.e. it is not an integral number.
 *
 * To check whether a number is an integer, use
 * [`Number.isInteger()`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/isInteger).
 */
export declare function isFloat(value: unknown): value is number;
/**
 * Returns true if the number is a safe number,
 * i.e. a number with a low chance of floating
 * point inaccuracies.
 *
 * This function tests for floating point
 * inaccuracy by conducting an arbitrary mathematical
 * operation on the value, inverting it, and comparing
 * it with the original value.
 *
 * *Note: This function may not always be 100% accurate.
 * Do not base any important algorithms on the result
 * of this function. If you're trying to compare two
 * floating point numbers, use `Numbers.compare()` instead.*
 */
export declare function isSafeFloat(value: number): boolean;
/**
 * Returns true if the number is an unsafe number,
 * i.e. a number with a high chance of floating
 * point inaccuracies.
 *
 * This function tests for floating point
 * inaccuracy by conducting an arbitrary mathematical
 * operation on the value, inverting it, and comparing
 * it with the original value.
 *
 * *Note: This function may not always be 100% accurate.
 * Do not base any important algorithms on the result
 * of this function. If you're trying to compare two
 * floating point numbers, use `Numbers.compare()` instead.*
 */
export declare function isUnsafeFloat(value: number): boolean;
/**
 * Determines the greatest common divisor of the integers. The
 * greatest common divisor is the largest positive integer
 * that divides each of the integers without a remainder. Uses the Euclidean
 * algorithm for computation.
 *
 * Returns 0 if any of the divisors are 0 or fractions.
 */
export declare function gcd(...values: number[]): number;
/**
 * Gets the least common multiple for the given integers.
 * The least common multiple is the smallest multiple of
 * themselves that all the integers share. For instance,
 * the least common multiple of `{3, 4, 5}` is `60` because
 * it is the first multiple that all the numbers can reach
 * (`3 * 20`, `4 * 15`, `5 * 12`).
 *
 * Returns 0 if any of the values are 0 or floating point numbers.
 */
export declare function lcm(...values: number[]): number;
/**
 * Gets the median of the specified numbers. The median is defined as:
 *
 * * If the sequence contains an odd number of values, the sequence
 *   is first sorted and then the middle value is extracted.
 * * If the sequence contains an even number of values, the sequence
 *   is first sorted and then the average of the two middle values is extracted.
 */
export declare function median(...values: number[]): number;
/** Gets the sum of the elements in the array. */
export declare function sum(...values: number[]): number;
/** Gets the mean of the elements in the array. */
export declare function mean(...values: number[]): number;
/**
 * Converts a positive non-zero integer to a roman numeral.
 *
 * This function ignores all decimal places, in case the number is not an integer.
 * If the number equals 0 or is less than 0, the function throws an error.
 */
export declare function roman(value: number): string;
/**
 * Returns the target number closest to the source number.
 *
 * If two target numbers have the same absolute difference to `source`,
 * returns the first one specified.
 *
 * If no target number is specified, returns `source`.
 */
export declare function closest(source: number, ...targets: number[]): number;
/**
 * Gets the exponent of a `base` raised to the given `power`.
 *
 * In other words, gets `n` in a formula of the shape:
 * `base raised to n = power`.
 */
export declare function exponent(base: number, power: number): number;
/**
 * Increments a counter that starts at an index (default: 0) and returns the
 * first integral number not found in the range.
 *
 * @param from A range of integers.
 * @param startIndex Optional. The index at which to start counting. Default: 0.
 */
export declare function firstUnique(from: Iterable<number>, startIndex?: number): number;
/**
 * Increments a counter that starts at an index (default: 0) and returns the
 * first integral number not found in the range returned by the property
 * callback.
 *
 * @param from An iterable of any type of objects
 * @param property A callback that is called once per element in the iterable
 *                 and should return an integer. The function will then return
 *                 the first positive integer not returned by this function.
 * @param startIndex Optional. The index at which to start counting. Default: 0.
 */
export declare function firstUnique<T>(from: Iterable<T>, property: ((object: T) => number) | number, startIndex?: number): number;
/**
 * Attempts to convert the value to a number and returns it if the conversion
 * succeeded. If the conversion failed, returns `undefined`.
 */
export declare function from(potentialNumber: unknown): number | undefined;
export {};
