import { Nullable } from '../types.js';
/**
 * Represents a value that may or may not have a value.
 * This class considers the following types "not a value":
 *
 * * `undefined`
 * * `null`
 * * `NaN`
 *
 * This type is inspired by Rust's
 * [Option](https://doc.rust-lang.org/std/option/enum.Option.html)
 * type and promotes null correctness by making it more difficult to
 * use nullable values in an unsafe manner. However, as this is an instance
 * type, this comes at some performance cost (mainly when creating/disposing
 * a large number of Options in a short amount of time). In performance-critical
 * code, it is therefore recommended to use the available static methods instead
 * of creating instances of this type.
 */
export default class Option<T> {
    protected readonly value: Nullable<T>;
    constructor(value?: Nullable<T>);
    /**
     * If this Option has a value, returns it,
     * otherwise throws an error.
     */
    unwrap(error?: Error | string): T;
    /** Returns `true` if this Option contains a value. */
    hasValue(): boolean;
    /**
     * Returns `true` if this Option contains a value that matches
     * the given predicate.
     */
    hasValue(predicate: (value: T) => boolean): boolean;
    /**
     * Returns `true` if the argument contains a value. For the context of
     * `Option`, values are all types except:
     *
     * * `undefined`
     * * `null`
     * * `NaN`
     */
    static hasValue<T>(value: Nullable<T>): value is NonNullable<T>;
    /**
     * Returns `true` if is Option contains `value`. This function compares
     * by reference. Use `Option.hasValue(value => ...)` to check for structural
     * equality.
     */
    contains(value: T): boolean;
    /**
     * If this Option has a value, returns it,
     * otherwise returns the value passed to this function.
     */
    or<U>(other: U): T | U;
    /**
     * If this Option has a value, returns it,
     * otherwise returns the result of the callback function.
     */
    orElse<U>(callback: () => U): T | U;
    /**
     * Creates an `Option<U>` from this `Option<T>` by mapping a non-empty
     * value using the passed transformer function and leaving an empty value
     * as-is.
     */
    map<U>(mapFn: (value: T) => Nullable<U>): Option<U>;
    /**
     * Converts the passed `Nullable<T>` to a `Nullable<U>` by mapping a non-null
     * value using the passed transformer function and leaving an empty value
     * as-is.
     */
    static map<T, U>(value: Nullable<T>, mapFn: (value: T) => Nullable<U>): Nullable<U>;
    /** Runs the passed callback if and only if this Option contains a value. */
    run(callback: (value: T) => void): void;
    /** Runs the passed callback if and only if the passed `Nullable<T>` is not empty. */
    static run<T>(value: Nullable<T>, callback: (value: T) => void): void;
    toString(): string;
}
