import Option from './Option.js';
/**
 * Represents the result of an operation.
 * Possible values are `Success`, in which case the Result
 * contains a value, or `Failure`, in which case the Result
 * contains an error object or string.
 *
 * Loosely inspired by Rust's
 * [https://doc.rust-lang.org/std/result/enum.Result.html](Result)
 * type.
 */
interface Result<T = void> {
    /** Returns `true` if the Result contains a success. */
    ok(): this is OkResult<T>;
    /** Returns `true` if the Result contains a failure. */
    nok(): this is ErrorResult<T>;
    /**
     * Gets the contained value if this Result represents a success,
     * otherwise throws the contained error.
     */
    unwrap(): T;
    /**
     * Gets the contained value if this Result represents a success,
     * otherwise returns the passed fallback value.
     */
    or(fallback: T): T;
    /**
     * Maps this `Result<T>` to a `Result<U>` by applying a transformer
     * function on a successful value.
     *
     * The transformer function can either return the new value directly or
     * another `Result` which contains either the new value or an error.
     */
    map<U>(mapFn: (value: T) => U | Result<U>): Result<U>;
    /** Converts this {@link Result}\<T> to an {@link Option}\<T>. */
    toOption(): Option<T>;
    /** Converts this {@link Result}\<T> to a string. */
    toString(): string;
}
declare namespace Result {
    /** Creates a new successful Result from a value. */
    function ok(): OkResult;
    function ok<T = void>(value: T): OkResult<T>;
    /** Creates a new failed Result from an error. */
    function nok<T = void>(error: Error | string): ErrorResult<T>;
    /**
     * Attempts to execute a callback and wraps its result in a new `Result`.
     * The `Result` will contain an error if the callback threw an error.
     *
     * Note that, if the callback throws a non-error value, it will be coerced
     * to a string and wrapped in a new `Error`.
     */
    function from<T>(callback: () => T): Result<T>;
    /**
     * If all the given `Result`s are successes, returns a new successful
     * `Result` without a value.
     *
     * If any of the given `Result`s are failures, returns a new failed `Result`
     * with the given error.
     */
    function all<TError extends Error | string>(...args: [...results: Result[], error: TError]): Result<void>;
    /**
     * If any the given `Result`s are successes, returns a new successful
     * `Result` without a value.
     *
     * If all of the given `Result`s are failures, returns a new failed `Result`
     * with the given error.
     */
    function any<TError extends Error | string>(...args: [...results: Result[], error: TError]): Result<void>;
    /** Returns `true` if the passed value is a `Result`. */
    function is<T>(value: unknown): value is Result<T>;
    function is(value: unknown): value is Result;
}
declare class OkResult<T = void> implements Result<T> {
    private readonly value;
    constructor(value: T);
    ok(): boolean;
    nok(): boolean;
    unwrap(): T;
    or(): T;
    map<U>(mapFn: (value: T) => U | Result<U>): Result<U>;
    toOption(): Option<T>;
    toString(): string;
}
declare class ErrorResult<T = void> implements Result<T> {
    private readonly error;
    constructor(value: Error | string);
    ok(): boolean;
    nok(): boolean;
    unwrap(): never;
    or(fallback: T): T;
    map<U>(): Result<U>;
    toOption(): Option<T>;
    toString(): string;
}
export default Result;
