// TODO:
/* v8 ignore start */
import { func } from '../fluent/matchers.js';
import Option from './Option.js';
import { stringify } from './stringify.js';
import { structure } from './structure.js';
const struct = {
    ok: func(),
    nok: func(),
    unwrap: func(),
    or: func(),
    map: func(),
    toOption: func(),
    toString: func()
};
var Result;
(function (Result) {
    function ok(value) {
        return new OkResult(value);
    }
    Result.ok = ok;
    /** Creates a new failed Result from an error. */
    function nok(error) {
        return new ErrorResult(error);
    }
    Result.nok = nok;
    /**
     * Attempts to execute a callback and wraps its result in a new `Result`.
     * The `Result` will contain an error if the callback threw an error.
     *
     * Note that, if the callback throws a non-error value, it will be coerced
     * to a string and wrapped in a new `Error`.
     */
    function from(callback) {
        try {
            return Result.ok(callback());
        }
        catch (e) {
            const error = e instanceof Error ? e : new Error(stringify(e));
            return Result.nok(error);
        }
    }
    Result.from = from;
    /**
     * If all the given `Result`s are successes, returns a new successful
     * `Result` without a value.
     *
     * If any of the given `Result`s are failures, returns a new failed `Result`
     * with the given error.
     */
    function all(...args) {
        if (args.length < 2) {
            throw new Error(`Not enough arguments supplied to Result.all(). Expected at least 2 arguments but got: ${args.length}`);
        }
        const results = args.slice(0, -1);
        if (results.every(result => result.ok())) {
            return Result.ok();
        }
        const error = args[args.length - 1];
        return Result.nok(error);
    }
    Result.all = all;
    /**
     * If any the given `Result`s are successes, returns a new successful
     * `Result` without a value.
     *
     * If all of the given `Result`s are failures, returns a new failed `Result`
     * with the given error.
     */
    function any(...args) {
        if (args.length < 2) {
            throw new Error(`Not enough arguments supplied to Result.all(). Expected at least 2 arguments but got: ${args.length}`);
        }
        const results = args.slice(0, -1);
        if (results.some(result => result.ok())) {
            return Result.ok();
        }
        const error = args[args.length - 1];
        return Result.nok(error);
    }
    Result.any = any;
    function is(value) {
        return structure(value, struct);
    }
    Result.is = is;
})(Result || (Result = {}));
class OkResult {
    constructor(value) {
        this.value = value;
    }
    ok() {
        return true;
    }
    nok() {
        return false;
    }
    unwrap() {
        return this.value;
    }
    or() {
        return this.value;
    }
    map(mapFn) {
        const result = mapFn(this.value);
        if (Result.is(result)) {
            return result;
        }
        return new OkResult(result);
    }
    toOption() {
        return new Option(this.value);
    }
    toString() {
        return `Success(${stringify(this.value)})`;
    }
}
class ErrorResult {
    constructor(value) {
        this.error = value instanceof Error ? value : new Error(value);
    }
    ok() {
        return false;
    }
    nok() {
        return true;
    }
    unwrap() {
        throw this.error;
    }
    or(fallback) {
        return fallback;
    }
    map() {
        return this;
    }
    toOption() {
        return new Option();
    }
    toString() {
        return `Failure(${this.error.message})`;
    }
}
export default Result;
