import { Constructor, Equatable } from '../types.js';
import Lazy from './Lazy.js';
declare type RecursionMode = 'shallow' | 'deep';
/**
 * Performs a shallow or deep comparison of the two objects and returns a value
 * indicating whether the two objects are equal.
 *
 * This is really only meant for literals, but you can technically pass any
 * value here. It'll automatically fall back to a standard `===` comparison if
 * either of the values are not true objects.
 *
 * @param comparisonMode What kind of comparison to apply. Possible values are
 * 'shallow' (only compares the first level of keys and values; compares nested
 * objects by reference only) and 'deep' (recursively compares all levels of
 * keys and values, meaning it properly compares nested objects as well).
 * Default is 'shallow'.
 */
export declare function compare(object1: unknown, object2: unknown, comparisonMode?: RecursionMode): boolean;
/**
 * Swaps the values `source[from]` and `source[to]` and returns the original object.
 * Note that this will invoke setters.
 */
export declare function swap<T extends object>(source: T, from: keyof T, to: keyof T): T;
/**
 * Compares any object (including primitives) with one or multiple others and returns true if all are equal.
 *
 * This function will use `Equatable` to check for equality if the source object implements it, otherwise
 * this function will use a simple referential equality check (or structural equality check in the case of value types).
 */
export declare function equal(source: unknown, ...others: unknown[]): boolean;
/**
 * Attempts to inject a default `equals()` function into the specified object that iterates through all
 * its keys and compares it with the target object.
 *
 * Do not use this function with prototyped objects. This function does not compare inherited
 * properties or functions.
 *
 * The object must be extensible for this function to succeed. If it is not, this function will
 * throw an error.
 *
 * This function does nothing if the target object already contains an `equals` function.
 * If the target object contains an `equals` property that is not a function, this function
 * will throw an error.
 *
 * @returns The original object cast with `Equatable` implemented.
 */
export declare function equatable<T extends object>(source: T): Equatable & T;
declare type MappedType<T, U> = {
    [key in keyof T]: U;
};
/**
 * Creates a new object literal that maps each property of the original object
 * with a transform function.
 */
export declare function map<T extends object, U>(object: T, callback: (value: T[typeof key], key: keyof T) => U): MappedType<T, U>;
/**
 * Creates a new object literal with only the properties that match the given
 * predicate.
 */
export declare function filter<T extends object>(object: T, predicate: (value: T[typeof key], key: keyof T) => boolean): Partial<T>;
/**
 * Creates a new object literal with the given keys removed.
 * This is analogous to TypeScript's `Omit<T>` type.
 */
export declare function omit<T extends object, Keys extends keyof T>(object: T, ...which: Keys[]): Omit<T, Keys>;
/**
 * Creates a new object literal with only the given keys from the source object.
 * This is analogous to TypeScript's `Pick<T>` type.
 */
export declare function pick<T extends object, Keys extends keyof T>(object: T, ...which: Keys[]): Pick<T, Keys>;
/**
 * Gets the property's property descriptor if it exists on the object
 * or somewhere along its prototype chain, else `undefined`.
 *
 * Contrast
 * [`Object.getOwnPropertyDescriptor()`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/getOwnPropertyDescriptor),
 * which only searches the object itself without considering its prototype.
 */
export declare function getPropertyDescriptor(object: unknown, property: string | number | symbol): PropertyDescriptor | undefined;
/**
 * Returns true if the property exists on the target object and
 * can be written to (i.e. is not read-only).
 */
export declare function isWritable<Key extends string | number | symbol>(object: unknown, property: Key): object is {
    [key in Key]: unknown;
};
/**
 * Gets the constructor of `object` if `object` is a class instance,
 * otherwise returns `undefined`.
 *
 * Note that primitives, object literals, and basic arrays will all
 * return `undefined`.
 */
export declare function getConstructor<T>(object: T): Constructor<T> | undefined;
/**
 * Clones the passed object. This algorithm will attempt, in order:
 *
 * 1. If `object` is a primitive or `null`, returns `object`.
 * 2. If `object` has a `clone()` function, calls it and returns the result.
 * 3. If `object` is an array, clones the array. If `mode` is "deep", calls
 *    `clone()` on each array item.
 * 4. Creates a new object literal and assigns all properties of `object`
 *    to it before returning it. If `mode` is "deep", calls `clone()` on
 *    each property value first.
 *
 * Note that this function can throw an error, for instance if `object`
 * or one of its properties is an instance of a class constructor.
 * In that case you probably want to implement a `clone()` function
 * on the type.
 */
export declare function clone<T>(object: T, mode?: RecursionMode): T;
/**
 * Returns a new lazily evaluated value.
 *
 * Internally, this function simply calls the {@link Lazy} constructor.
 * If you'd prefer to use a `new` expression like `new Lazy(...)`,
 * you can use the {@link Lazy} class directly.
 *
 * @param valueCallback A callback that is executed to retrieve a value
 * when it is requested for the first time.
 */
export declare function lazy<T>(valueCallback: () => T): Lazy<T>;
export * from './stringify.js';
export * from './typechecking.js';
export * from './structure.js';
