/** Options for `Strings.serialize()`. */
export interface StringifyOptions {
    /**
     * Whether the contents of arrays and objects should be truncated
     * and replaced with an ellipsis (`...`).
     *
     * You can also specify a number here to specify how many properties
     * or array elements you want to show before the remaining ones are
     * truncated.
     *
     * Default is `false`.
     */
    truncateContents?: boolean | number;
    /**
     * If true, stringify will only print the types, not their values.
     * For instance, the value `4` would be replaced by `number`.
     *
     * This property overrides `useToString`.
     *
     * Default is `false`.
     */
    typesOnly?: boolean;
    /**
     * Like `typesOnly`, except that this property affects only primitives.
     * Objects and arrays will be printed like normal.
     *
     * This property is overridden by `typesOnly`.
     *
     * Default is `false`.
     */
    primitiveTypesOnly?: boolean;
    /**
     * By default if a non-primitive (that is, an Object, Array, or Function)
     * is an instance of a non-standard class, its class name is printed before
     * the main expression. If this property is true, the class name will
     * be omitted even when the type is non-standard.
     *
     * This property is overridden by `typesOnly`.
     *
     * Default is `false`.
     */
    hideClasses?: boolean;
    /**
     * If true, this function uses a type's `toString()` function if and only if
     * it has a custom implementation, i.e. it does not return the standard
     * "`[object Type]`".
     *
     * This property is overridden by `typesOnly`.
     *
     * Default is `true`.
     */
    useToString?: boolean;
    /**
     * By default, stringify() adds double quotes to either side of natural
     * strings. This property removes those quotes.
     *
     * Default is `false`.
     */
    omitQuotes?: boolean;
    /**
     * If specified, this function will be called for every value that is to
     * be converted to string (i.e. the main value, all properties, and all
     * array elements).
     *
     * If the function returns a string, that string is used instead of the
     * default string conversion of stringify() for that value. If this function
     * returns `null`, the default string conversion is used instead. This
     * allows you to supply your own string converters selectively for
     * certain types.
     */
    replacer?: (value: unknown) => string | null;
}
/**
 * The symbol used to implement custom `Objects.stringify()` behaviour
 * for objects. See the {@link Stringifiable} interface.
 */
export declare const Stringify: unique symbol;
/**
 * Represents an object that has a custom {@link stringify}() implementation.
 * potence defines a default `stringify()` implementation for {@link Object},
 * {@link Array}, and {@link Function}. You can override these implementations
 * with your own by defining a new {@link Stringify} function on those types'
 * prototypes, or give your own types a custom {@link stringify}()
 * implementation by implementing this interface.
 *
 * ### On `toString()`
 * Note that `toString()` is a default function of many objects and is used
 * to convert an object into a meaningful textual form that can be understood
 * by a user.
 *
 * In contrast, the {@link Stringify} method is solely called by
 * {@link stringify}() and is primarily intended to convert types to debug
 * output that can be interpreted by a developer.
 */
export interface Stringifiable {
    [Stringify](options: StringifyOptions): string;
}
/**
 * Converts an arbitrary value to string. See `StringifyOptions` for
 * information on how to customize this function.
 *
 * This function was primarily designed for the purposes of readable
 * and expressive debug output (for instance, the `assert` module
 * heavily utilizes it), but it can be used for other purposes as well.
 *
 * If a type implements {@link Stringifiable} (which utilizes the
 * {@link Stringify} symbol), it will be used to convert that type
 * to a string representation here.
 */
export declare function stringify(value: unknown, options?: StringifyOptions): string;
