import { ObjectLiteral, TypeofResult, Constructor, Typeof, Equatable, primitive, Func } from '../types.js';
/**
 * Checks if the passed object is of the specified type.
 *
 * Depending on the type of the second argument, this function
 * will either call `typeof` or `instanceof`.
 */
export declare function is<T>(object: unknown, type: Constructor<T>): object is T;
export declare function is<TTypeOf extends TypeofResult>(object: unknown, type: TTypeOf): object is TTypeOf;
/**
 * Checks if a value is an object, excluding `null` but including arrays and functions.
 * If this check succeeds, you can safely access the object's members via bracket syntax.
 */
export declare function isObject(value: unknown): value is ObjectLiteral;
/**
 * Checks if a value is an object literal, i.e. an object whose direct prototype is `Object.prototype`.
 * Like `isObject()`, you can safely access the object's members via bracket syntax if this check succeeds.
 */
export declare function isObjectLiteral(value: unknown): value is ObjectLiteral;
/** Checks if a given object is a primitive, i.e. if it is not a function, object, array, or instance. */
export declare function isPrimitive(object: unknown): object is primitive;
/** Checks if an object implements `Equatable`. */
export declare function isEquatable(object: unknown): object is Equatable;
/** Checks if an object implements `Iterable`. */
export declare function isIterable(object: unknown): object is Iterable<unknown>;
declare type HasProperty<TKey extends string | number | symbol, T> = {
    [key in TKey]: T;
};
declare type HasPropertyReturnType<T extends TypeofResult | Constructor> = T extends TypeofResult ? Typeof<T> : T extends Constructor<infer C> ? C : unknown;
/**
 * Checks if an object has a property with the specified name.
 * Optionally checks if the property is the specified type.
 */
export declare function hasProperty<TKey extends string | number | symbol, T extends TypeofResult | Constructor>(source: unknown, propertyName: TKey, type?: T): source is HasProperty<TKey, HasPropertyReturnType<T>>;
/**
 * Checks if an object has a function with the specified name
 * and optionally the specified number of arguments.
 *
 * Note that `argumentCount` corresponds to the function's *formal parameters*,
 * i.e. the exact number of parameters defined in the function signature,
 * excluding any rest parameters. This also includes *optional* parameters.
 */
export declare function hasFunction<T extends string | number | symbol>(source: unknown, functionName: T): source is HasProperty<T, Func>;
export declare function hasFunction<T extends string | number | symbol, N extends number>(source: unknown, functionName: T, argumentCount: N): source is HasProperty<T, Func<N>>;
/**
 * Attempts to cast the object to the specified type. If the object is assignable
 * to the type, returns the object, otherwise returns `undefined`.
 */
export declare function cast<T>(object: unknown, type: Constructor<T>): T | undefined;
export declare function cast<TTypeOf extends TypeofResult>(object: unknown, type: TTypeOf): Typeof<TTypeOf> | undefined;
/**
 * Attempts to cast the object to the specified type. If the object is assignable
 * to the type, returns the object, otherwise throws an error.
 */
export declare function hardCast<T>(object: unknown, type: Constructor<T>): T;
export declare function hardCast<TTypeOf extends TypeofResult>(object: unknown, type: TTypeOf): Typeof<TTypeOf>;
export {};
