export function is(object, type) {
    return typeof type === 'string' ? typeof object === type : object instanceof type;
}
/**
 * Checks if a value is an object, excluding `null` but including arrays and functions.
 * If this check succeeds, you can safely access the object's members via bracket syntax.
 */
export function isObject(value) {
    return value != null && (typeof value === 'object' || typeof value === 'function');
}
/**
 * Checks if a value is an object literal, i.e. an object whose direct prototype is `Object.prototype`.
 * Like `isObject()`, you can safely access the object's members via bracket syntax if this check succeeds.
 */
export function isObjectLiteral(value) {
    return value != null && typeof value === 'object' && Object.getPrototypeOf(value) === Object.prototype;
}
/** Checks if a given object is a primitive, i.e. if it is not a function, object, array, or instance. */
export function isPrimitive(object) {
    return object !== null && typeof object !== 'object' && typeof object !== 'function';
}
/** Checks if an object implements `Equatable`. */
export function isEquatable(object) {
    return object != null && hasFunction(object, 'equals', 1);
}
/** Checks if an object implements `Iterable`. */
export function isIterable(object) {
    return object != null && hasFunction(object, Symbol.iterator);
}
/**
 * Checks if an object has a property with the specified name.
 * Optionally checks if the property is the specified type.
 */
export function hasProperty(source, propertyName, type) {
    if (source == null) {
        return false;
    }
    // We use the 'in' operator to distinguish between properties whose
    // value is undefined and properties that don't actually exist.
    // However, the 'in' operator can only be used on objects,
    // not primitives. But this is not a problem because primitives
    // don't have any properties whose value could be undefined.
    const object = source;
    if (isPrimitive(source) ? object[propertyName] === undefined : !(propertyName in object)) {
        return false;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    return (type == null || is(object[propertyName], type));
}
export function hasFunction(source, functionName, argumentCount) {
    return source != null
        && typeof source[functionName] === 'function'
        && (argumentCount == null
            // eslint-disable-next-line @typescript-eslint/ban-types
            || source[functionName].length === argumentCount);
}
export function cast(object, type) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    if (is(object, type)) {
        return object;
    }
    return undefined;
}
export function hardCast(object, type) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    if (is(object, type)) {
        return object;
    }
    throw new Error(`Cast failed. ${object} is not assignable to type ${type}.`);
}
