import { Equatable } from '../types.js';
export interface UnitsStringOptions {
    /** The amount of whitespace after each unit specifier. Default is `0`. */
    spaces: number;
    /**
     * Whether to include the number of milliseconds in the output or not, if
     * there are more than 0 ms remaining. Default is `true`.
     */
    includeMilliseconds: boolean;
}
/**
 * Represents a span of time.
 *
 * The smallest unit of time that may be represented by this class is
 * milliseconds.
 */
export default class Duration implements Equatable {
    private readonly value;
    /**
     * Parses a Duration from a duration string in the format of
     * `ww:dd:hh:mm:ss.ms`. Note that the only required components are `mm:ss`,
     * the other components are optional.
     */
    constructor(ms: string);
    /**
     * Creates a new Duration instance from the given number of milliseconds.
     * This function is identical to {@link Duration.fromMilliseconds}();
     */
    constructor(ms: number);
    /** Gets the total fractional number of milliseconds in this {@link Duration}. */
    totalMilliseconds(): number;
    /** Gets the total fractional number of seconds in this {@link Duration}. */
    totalSeconds(): number;
    /** Gets the total fractional number of minutes in this {@link Duration}. */
    totalMinutes(): number;
    /** Gets the total fractional number of hours in this {@link Duration}. */
    totalHours(): number;
    /** Gets the total fractional number of days in this {@link Duration}. */
    totalDays(): number;
    /**
     * Gets the milliseconds component of this {@link Duration} as an integral
     * number between `0` and `999`.
     */
    milliseconds(): number;
    /**
     * Gets the seconds component of this {@link Duration} as an integral
     * number between `0` and `59`.
     */
    seconds(): number;
    /**
     * Gets the minutes component of this {@link Duration} as an integral
     * number between `0` and `999`.
     */
    minutes(): number;
    /**
     * Gets the hours component of this {@link Duration} as an integral
     * number between `0` and `23`.
     */
    hours(): number;
    /**
     * Gets the days component of this {@link Duration} as an integral
     * number.
     */
    days(): number;
    /** Adds the given {@link Duration} to a copy of this duration. */
    add(duration: Duration): Duration;
    /** Subtracts the given {@link Duration} from a copy of this duration. */
    sub(duration: Duration): Duration;
    /** Multiplies this {@link Duration} by the given factor. */
    multiply(by: number): Duration;
    /** Divides this {@link Duration} by the given factor. */
    divide(by: number): Duration;
    equals(duration: unknown): boolean;
    /**
     * Returns a string representation of this {@link Duration}, for example
     * `03:20:15.000`.
     *
     * See {@link toUnitsString}() for a format like `2h54s`.
     *
     * @param format The format to use. Is `dd:hh:mm:ss.000` by default
     * where `000` stands for the number of milliseconds. The number of repeated
     * characters indicates the number of leading zeroes.
     * Placeholders that evaluate to `0` are automatically omitted at the front
     * of the string.
     */
    toString(format?: string): string;
    /**
     * Returns a string representation of this {@link Duration}
     * in the format of `1w2d3h4m5s6ms` where empty components are
     * omitted.
     *
     * @param spaces Sets the amount of whitespace after each unit
     * identifier. Default is `0`.
     */
    toUnitsString(options?: UnitsStringOptions): string;
    /**
     * Creates a new {@link Duration} from a number of milliseconds.
     */
    static fromMilliseconds(value: number): Duration;
    /**
     * Creates a new {@link Duration} from a number of seconds.
     */
    static fromSeconds(value: number): Duration;
    /**
     * Creates a new {@link Duration} from a number of minutes.
     */
    static fromMinutes(value: number): Duration;
    /**
     * Creates a new {@link Duration} from a number of hours.
     */
    static fromHours(value: number): Duration;
    /**
     * Creates a new {@link Duration} from a number of days.
     */
    static fromDays(value: number): Duration;
    /**
     * Gets the difference between two {@link Date}s or their contained values.
     *
     * Note that, because {@link Duration}s cannot be negative, the order
     * of the operands does not matter. Use `Time.compare()`
     * to check which of the two comes first.
     */
    static difference(a: Date | number, b: Date | number): Duration;
    /**
     * Gets the duration that has elapsed since the given {@link Date}.
     */
    static elapsed(date: Date): Duration;
}
